"""
Layer Communication Demo - 层间通信协议使用示例
演示7层架构间的通信机制
"""

import asyncio
import logging
import time
from datetime import datetime
from typing import Dict, Any

# 导入层间通信协议
from layers.framework.abstractions.layer_communication import (
    LayerCommunicationManager,
    LayerRequest,
    LayerResponse,
    ExecutionContext,
    MessageType,
    LayerEventHandler
)

# 设置日志
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


class BusinessLayerHandler:
    """业务能力层处理器"""
    
    def __init__(self):
        self.communicator = LayerCommunicationManager().get_communicator("business")
        self._register_handlers()
    
    def _register_handlers(self):
        """注册请求处理器"""
        self.communicator.register_request_handler("create_workflow", self._handle_create_workflow)
        self.communicator.register_request_handler("execute_collaboration", self._handle_execute_collaboration)
    
    async def _handle_create_workflow(self, payload: Dict[str, Any], context: ExecutionContext) -> Dict[str, Any]:
        """处理创建工作流请求"""
        logger.info(f"Business layer: Creating workflow with parameters: {payload['parameters']}")
        
        # 模拟工作流创建
        workflow_id = f"workflow_{int(time.time())}"
        
        # 发布工作流创建事件
        await self.communicator.publish_event(
            "workflow_created",
            {"workflow_id": workflow_id, "parameters": payload['parameters']},
            context
        )
        
        return {"workflow_id": workflow_id, "status": "created"}
    
    async def _handle_execute_collaboration(self, payload: Dict[str, Any], context: ExecutionContext) -> Dict[str, Any]:
        """处理执行协作请求"""
        logger.info(f"Business layer: Executing collaboration with pattern: {payload['parameters']['pattern']}")
        
        # 调用认知架构层进行智能决策
        cognitive_request = LayerRequest(
            operation="analyze_collaboration",
            parameters={
                "pattern": payload['parameters']['pattern'],
                "participants": payload['parameters']['participants']
            }
        )
        
        response = await self.communicator.send_request(
            "cognitive",
            cognitive_request,
            context
        )
        
        if response.success:
            return {
                "collaboration_id": f"collab_{int(time.time())}",
                "analysis_result": response.data,
                "status": "executing"
            }
        else:
            return {"error": response.error}


class CognitiveLayerHandler:
    """认知架构层处理器"""
    
    def __init__(self):
        self.communicator = LayerCommunicationManager().get_communicator("cognitive")
        self._register_handlers()
    
    def _register_handlers(self):
        """注册请求处理器"""
        self.communicator.register_request_handler("analyze_collaboration", self._handle_analyze_collaboration)
        self.communicator.register_request_handler("reason_about_task", self._handle_reason_about_task)
    
    async def _handle_analyze_collaboration(self, payload: Dict[str, Any], context: ExecutionContext) -> Dict[str, Any]:
        """处理协作分析请求"""
        logger.info(f"Cognitive layer: Analyzing collaboration pattern: {payload['parameters']['pattern']}")
        
        # 模拟认知分析
        analysis_result = {
            "optimal_strategy": "sequential_execution",
            "estimated_duration": 120,
            "resource_requirements": {"cpu": 2, "memory": "4GB"},
            "confidence_score": 0.85
        }
        
        # 发布认知分析完成事件
        await self.communicator.publish_event(
            "collaboration_analyzed",
            {"analysis_result": analysis_result},
            context
        )
        
        return analysis_result
    
    async def _handle_reason_about_task(self, payload: Dict[str, Any], context: ExecutionContext) -> Dict[str, Any]:
        """处理任务推理请求"""
        logger.info(f"Cognitive layer: Reasoning about task: {payload['parameters']['task_description']}")
        
        # 调用框架抽象层获取Agent能力
        framework_request = LayerRequest(
            operation="get_agent_capabilities",
            parameters={"agent_type": payload['parameters']['agent_type']}
        )
        
        response = await self.communicator.send_request(
            "framework",
            framework_request,
            context
        )
        
        if response.success:
            # 基于Agent能力进行推理
            reasoning_result = {
                "task_complexity": "medium",
                "required_capabilities": response.data,
                "execution_plan": ["perception", "reasoning", "action"],
                "confidence": 0.92
            }
            return reasoning_result
        else:
            return {"error": "Failed to get agent capabilities"}


class FrameworkLayerHandler:
    """框架抽象层处理器"""
    
    def __init__(self):
        self.communicator = LayerCommunicationManager().get_communicator("framework")
        self._register_handlers()
    
    def _register_handlers(self):
        """注册请求处理器"""
        self.communicator.register_request_handler("get_agent_capabilities", self._handle_get_agent_capabilities)
        self.communicator.register_request_handler("create_agent", self._handle_create_agent)
    
    async def _handle_get_agent_capabilities(self, payload: Dict[str, Any], context: ExecutionContext) -> Dict[str, Any]:
        """处理获取Agent能力请求"""
        agent_type = payload['parameters']['agent_type']
        logger.info(f"Framework layer: Getting capabilities for agent type: {agent_type}")
        
        # 模拟Agent能力查询
        capabilities = {
            "openai": ["conversation", "code_generation", "analysis"],
            "autogen": ["multi_agent_collaboration", "task_decomposition"],
            "custom": ["domain_specific", "custom_tools"]
        }
        
        return capabilities.get(agent_type, [])
    
    async def _handle_create_agent(self, payload: Dict[str, Any], context: ExecutionContext) -> Dict[str, Any]:
        """处理创建Agent请求"""
        logger.info(f"Framework layer: Creating agent with config: {payload['parameters']}")
        
        # 调用适配器层创建具体Agent
        adapter_request = LayerRequest(
            operation="create_agent_instance",
            parameters=payload['parameters']
        )
        
        response = await self.communicator.send_request(
            "adapter",
            adapter_request,
            context
        )
        
        if response.success:
            return {
                "agent_id": response.data['agent_id'],
                "agent_type": payload['parameters']['agent_type'],
                "status": "created"
            }
        else:
            return {"error": response.error}


class AdapterLayerHandler:
    """适配器层处理器"""
    
    def __init__(self):
        self.communicator = LayerCommunicationManager().get_communicator("adapter")
        self._register_handlers()
    
    def _register_handlers(self):
        """注册请求处理器"""
        self.communicator.register_request_handler("create_agent_instance", self._handle_create_agent_instance)
        self.communicator.register_request_handler("execute_task", self._handle_execute_task)
    
    async def _handle_create_agent_instance(self, payload: Dict[str, Any], context: ExecutionContext) -> Dict[str, Any]:
        """处理创建Agent实例请求"""
        logger.info(f"Adapter layer: Creating agent instance: {payload['parameters']['agent_type']}")
        
        # 调用基础设施层获取资源
        infrastructure_request = LayerRequest(
            operation="allocate_resources",
            parameters={
                "resource_type": "agent_instance",
                "requirements": payload['parameters'].get('requirements', {})
            }
        )
        
        response = await self.communicator.send_request(
            "infrastructure",
            infrastructure_request,
            context
        )
        
        if response.success:
            agent_id = f"agent_{payload['parameters']['agent_type']}_{int(time.time())}"
            return {
                "agent_id": agent_id,
                "allocated_resources": response.data,
                "status": "ready"
            }
        else:
            return {"error": "Failed to allocate resources"}
    
    async def _handle_execute_task(self, payload: Dict[str, Any], context: ExecutionContext) -> Dict[str, Any]:
        """处理执行任务请求"""
        logger.info(f"Adapter layer: Executing task: {payload['parameters']['task']}")
        
        # 模拟任务执行
        await asyncio.sleep(0.1)  # 模拟执行时间
        
        return {
            "task_id": f"task_{int(time.time())}",
            "result": f"Task completed: {payload['parameters']['task']}",
            "execution_time": 0.1
        }


class InfrastructureLayerHandler:
    """基础设施层处理器"""
    
    def __init__(self):
        self.communicator = LayerCommunicationManager().get_communicator("infrastructure")
        self._register_handlers()
    
    def _register_handlers(self):
        """注册请求处理器"""
        self.communicator.register_request_handler("allocate_resources", self._handle_allocate_resources)
        self.communicator.register_request_handler("get_config", self._handle_get_config)
    
    async def _handle_allocate_resources(self, payload: Dict[str, Any], context: ExecutionContext) -> Dict[str, Any]:
        """处理资源分配请求"""
        logger.info(f"Infrastructure layer: Allocating resources: {payload['parameters']}")
        
        # 模拟资源分配
        allocated_resources = {
            "cpu_cores": 2,
            "memory_mb": 4096,
            "storage_gb": 10,
            "network_bandwidth": "100Mbps"
        }
        
        # 发布资源分配事件
        await self.communicator.publish_event(
            "resources_allocated",
            {"allocated_resources": allocated_resources},
            context
        )
        
        return allocated_resources
    
    async def _handle_get_config(self, payload: Dict[str, Any], context: ExecutionContext) -> Dict[str, Any]:
        """处理配置获取请求"""
        logger.info(f"Infrastructure layer: Getting config for: {payload['parameters']['config_key']}")
        
        # 模拟配置获取
        configs = {
            "openai": {"api_key": "***", "model": "gpt-4"},
            "database": {"url": "postgresql://localhost/adc"},
            "cache": {"redis_url": "redis://localhost:6379"}
        }
        
        config_key = payload['parameters']['config_key']
        return configs.get(config_key, {})


class EventMonitor(LayerEventHandler):
    """事件监控器 - 监听所有层间事件"""
    
    def __init__(self):
        self.events_received = []
    
    async def handle_event(self, event, context: ExecutionContext) -> None:
        """处理层间事件"""
        event_info = {
            "timestamp": datetime.now().isoformat(),
            "source_layer": event.source_layer,
            "event_type": event.payload.get("event_type"),
            "data": event.payload.get("data", {}),
            "trace_id": event.trace_id
        }
        self.events_received.append(event_info)
        logger.info(f"Event received: {event_info}")


async def demo_layer_communication():
    """演示层间通信"""
    logger.info("=== 层间通信协议演示 ===")
    
    # 初始化各层处理器
    business_handler = BusinessLayerHandler()
    cognitive_handler = CognitiveLayerHandler()
    framework_handler = FrameworkLayerHandler()
    adapter_handler = AdapterLayerHandler()
    infrastructure_handler = InfrastructureLayerHandler()
    
    # 创建事件监控器
    event_monitor = EventMonitor()
    
    # 订阅事件
    for handler in [business_handler, cognitive_handler, framework_handler, adapter_handler, infrastructure_handler]:
        handler.communicator.subscribe_to_events("*", event_monitor)
    
    # 创建执行上下文
    context = ExecutionContext(
        request_id=f"req_{int(time.time())}",
        user_id="demo_user",
        session_id="demo_session",
        project_id="demo_project"
    )
    
    # 演示1: 创建协作工作流
    logger.info("\n--- 演示1: 创建协作工作流 ---")
    workflow_request = LayerRequest(
        operation="create_workflow",
        parameters={
            "name": "数据分析工作流",
            "steps": ["数据收集", "数据清洗", "数据分析", "报告生成"]
        }
    )
    
    response = await business_handler.communicator.send_request(
        "business",
        workflow_request,
        context
    )
    
    logger.info(f"工作流创建结果: {response.data}")
    
    # 演示2: 执行多Agent协作
    logger.info("\n--- 演示2: 执行多Agent协作 ---")
    collaboration_request = LayerRequest(
        operation="execute_collaboration",
        parameters={
            "pattern": "expert_consultation",
            "participants": ["data_analyst", "domain_expert", "report_writer"]
        }
    )
    
    response = await business_handler.communicator.send_request(
        "business",
        collaboration_request,
        context
    )
    
    logger.info(f"协作执行结果: {response.data}")
    
    # 演示3: 创建Agent
    logger.info("\n--- 演示3: 创建Agent ---")
    agent_request = LayerRequest(
        operation="create_agent",
        parameters={
            "agent_type": "openai",
            "name": "数据分析助手",
            "capabilities": ["data_analysis", "report_generation"]
        }
    )
    
    response = await framework_handler.communicator.send_request(
        "framework",
        agent_request,
        context
    )
    
    logger.info(f"Agent创建结果: {response.data}")
    
    # 显示事件统计
    logger.info(f"\n--- 事件统计 ---")
    logger.info(f"总共接收到 {len(event_monitor.events_received)} 个事件:")
    for event in event_monitor.events_received:
        logger.info(f"  - {event['timestamp']}: {event['source_layer']} -> {event['event_type']}")
    
    # 显示执行上下文
    logger.info(f"\n--- 执行上下文 ---")
    logger.info(f"执行栈: {' -> '.join(context.execution_stack)}")
    logger.info(f"层执行时间: {context.layer_timings}")
    logger.info(f"错误数量: {len(context.errors)}")
    logger.info(f"警告数量: {len(context.warnings)}")


if __name__ == "__main__":
    asyncio.run(demo_layer_communication()) 


"""
CLI Commands
命令行界面命令系统 - 增强版
"""

import argparse
import asyncio
import os
import sys
import json
import yaml
import shutil
from pathlib import Path
from typing import Dict, Any, Callable, Optional, List
from datetime import datetime
import subprocess
import tempfile
import webbrowser

# 添加项目根目录到路径
project_root = Path(__file__).parent.parent.parent.parent
sys.path.insert(0, str(project_root))

try:
    from rich.console import Console
    from rich.table import Table
    from rich.progress import track, Progress, SpinnerColumn, TextColumn
    from rich.panel import Panel
    from rich.syntax import Syntax
    from rich.tree import Tree
    from rich import print as rprint
    RICH_AVAILABLE = True
except ImportError:
    RICH_AVAILABLE = False

from ...adapter.openai.adapter import OpenAIAdapter
from ...adapter.openai.agent_wrapper import OpenAIAgentWrapper
from ...business.teams.collaboration_manager import CollaborationManager, CollaborationPattern, CollaborationRole
from ...business.workflows.workflow_engine import WorkflowEngine, WorkflowStepType
from ...framework.abstractions.task import UniversalTask, TaskType
from ...framework.abstractions.context import UniversalContext
from ...framework.abstractions.agent import AgentCapability


class CommandRegistry:
    """
    命令注册系统 - 增强版
    """
    
    def __init__(self):
        self.commands: Dict[str, Callable] = {}
        if RICH_AVAILABLE:
            self.console = Console()
        self.project_root = Path.cwd()
        
    def register_commands(self, subparsers) -> None:
        """注册所有命令"""
        
        # Agent命令组
        agent_parser = subparsers.add_parser('agent', help='🤖 Agent管理命令')
        agent_subparsers = agent_parser.add_subparsers(dest='agent_action')
        
        # agent list
        list_parser = agent_subparsers.add_parser('list', help='📋 列出所有Agent')
        list_parser.add_argument('--format', choices=['table', 'json', 'yaml'], default='table', help='输出格式')
        list_parser.add_argument('--filter', help='过滤条件（名称、类型等）')
        list_parser.add_argument('--sort', choices=['name', 'type', 'created'], default='name', help='排序字段')
        
        # agent create
        create_parser = agent_subparsers.add_parser('create', help='✨ 创建新Agent')
        create_parser.add_argument('--name', required=True, help='Agent名称')
        create_parser.add_argument('--type', choices=['openai', 'autogen', 'langgraph', 'fpga_expert', 'code_expert', 'data_analyst', 'custom'], default='openai', help='Agent类型')
        create_parser.add_argument('--model', default='gpt-4o-mini', help='使用的模型')
        create_parser.add_argument('--system-message', help='系统消息')
        create_parser.add_argument('--template', choices=['basic', 'advanced', 'enterprise', 'fpga_basic', 'fpga_advanced', 'verification_expert'], help='使用模板创建')
        create_parser.add_argument('--capabilities', nargs='+', help='Agent能力列表')
        create_parser.add_argument('--interactive', action='store_true', help='交互式创建')
        
        # agent chat
        chat_parser = agent_subparsers.add_parser('chat', help='💬 与Agent对话')
        chat_parser.add_argument('--name', required=True, help='Agent名称')
        chat_parser.add_argument('--message', help='消息内容（如果不提供则进入交互模式）')
        chat_parser.add_argument('--history', action='store_true', help='显示对话历史')
        chat_parser.add_argument('--save', help='保存对话到文件')
        
        # agent info
        info_parser = agent_subparsers.add_parser('info', help='ℹ️ 显示Agent详细信息')
        info_parser.add_argument('name', help='Agent名称')
        
        # agent delete
        delete_parser = agent_subparsers.add_parser('delete', help='🗑️ 删除Agent')
        delete_parser.add_argument('name', help='Agent名称')
        delete_parser.add_argument('--force', action='store_true', help='强制删除（不确认）')
        
        # Workflow命令组
        workflow_parser = subparsers.add_parser('workflow', help='⚙️ 工作流管理命令')
        workflow_subparsers = workflow_parser.add_subparsers(dest='workflow_action')
        
        # workflow list
        wf_list_parser = workflow_subparsers.add_parser('list', help='📋 列出所有工作流')
        wf_list_parser.add_argument('--format', choices=['table', 'json', 'yaml'], default='table', help='输出格式')
        wf_list_parser.add_argument('--status', choices=['active', 'completed', 'failed'], help='按状态过滤')
        
        # workflow create
        wf_create_parser = workflow_subparsers.add_parser('create', help='✨ 创建工作流')
        wf_create_parser.add_argument('--name', required=True, help='工作流名称')
        wf_create_parser.add_argument('--description', help='工作流描述')
        wf_create_parser.add_argument('--template', help='使用模板')
        wf_create_parser.add_argument('--file', help='从文件创建工作流')
        wf_create_parser.add_argument('--interactive', action='store_true', help='交互式创建')
        
        # workflow run
        wf_run_parser = workflow_subparsers.add_parser('run', help='▶️ 运行工作流')
        wf_run_parser.add_argument('--id', required=True, help='工作流ID')
        wf_run_parser.add_argument('--context', help='初始上下文（JSON格式）')
        wf_run_parser.add_argument('--watch', action='store_true', help='实时监控执行状态')
        wf_run_parser.add_argument('--timeout', type=int, default=300, help='超时时间（秒）')
        
        # workflow status
        wf_status_parser = workflow_subparsers.add_parser('status', help='📊 查看工作流状态')
        wf_status_parser.add_argument('id', help='工作流执行ID')
        wf_status_parser.add_argument('--follow', action='store_true', help='持续跟踪状态')
        
        # workflow logs
        wf_logs_parser = workflow_subparsers.add_parser('logs', help='📄 查看工作流日志')
        wf_logs_parser.add_argument('id', help='工作流执行ID')
        wf_logs_parser.add_argument('--follow', action='store_true', help='持续跟踪日志')
        wf_logs_parser.add_argument('--lines', type=int, default=100, help='显示行数')
        
        # Team命令组
        team_parser = subparsers.add_parser('team', help='👥 团队管理命令')
        team_subparsers = team_parser.add_subparsers(dest='team_action')
        
        # team list
        team_list_parser = team_subparsers.add_parser('list', help='📋 列出所有团队')
        team_list_parser.add_argument('--format', choices=['table', 'json', 'yaml'], default='table', help='输出格式')
        
        # team create
        team_create_parser = team_subparsers.add_parser('create', help='✨ 创建团队')
        team_create_parser.add_argument('--name', required=True, help='团队名称')
        team_create_parser.add_argument('--members', nargs='+', help='团队成员列表')
        team_create_parser.add_argument('--template', help='使用团队模板')
        team_create_parser.add_argument('--interactive', action='store_true', help='交互式创建')
        
        # team collaborate
        team_collab_parser = team_subparsers.add_parser('collaborate', help='🤝 执行团队协作')
        team_collab_parser.add_argument('--team', required=True, help='团队名称')
        team_collab_parser.add_argument('--task', required=True, help='任务描述')
        team_collab_parser.add_argument('--pattern', 
                                      choices=[p.value for p in CollaborationPattern],
                                      default='parallel', help='协作模式')
        team_collab_parser.add_argument('--watch', action='store_true', help='实时监控协作状态')
        
        # team performance
        team_perf_parser = team_subparsers.add_parser('performance', help='📊 查看团队性能')
        team_perf_parser.add_argument('name', help='团队名称')
        team_perf_parser.add_argument('--period', choices=['day', 'week', 'month'], default='week', help='统计周期')
        
        # Project命令组
        project_parser = subparsers.add_parser('project', help='🏗️ 项目管理命令')
        project_subparsers = project_parser.add_subparsers(dest='project_action')
        
        # project init
        proj_init_parser = project_subparsers.add_parser('init', help='🚀 初始化新项目')
        proj_init_parser.add_argument('name', help='项目名称')
        proj_init_parser.add_argument('--template', help='项目模板')
        proj_init_parser.add_argument('--path', help='项目路径')
        proj_init_parser.add_argument('--interactive', action='store_true', help='交互式创建')
        
        # project list
        proj_list_parser = project_subparsers.add_parser('list', help='📋 列出所有项目')
        proj_list_parser.add_argument('--format', choices=['table', 'json', 'yaml'], default='table', help='输出格式')
        
        # project status
        proj_status_parser = project_subparsers.add_parser('status', help='📊 查看项目状态')
        proj_status_parser.add_argument('name', nargs='?', help='项目名称（默认当前项目）')
        
        # project build
        proj_build_parser = project_subparsers.add_parser('build', help='🔨 构建项目')
        proj_build_parser.add_argument('--target', help='构建目标')
        proj_build_parser.add_argument('--watch', action='store_true', help='监控文件变化并自动构建')
        
        # project deploy
        proj_deploy_parser = project_subparsers.add_parser('deploy', help='🚀 部署项目')
        proj_deploy_parser.add_argument('--env', choices=['dev', 'staging', 'prod'], default='dev', help='部署环境')
        proj_deploy_parser.add_argument('--dry-run', action='store_true', help='预演模式')
        
        # Config命令组
        config_parser = subparsers.add_parser('config', help='⚙️ 配置管理命令')
        config_subparsers = config_parser.add_subparsers(dest='config_action')
        
        # config get
        config_get_parser = config_subparsers.add_parser('get', help='📖 获取配置值')
        config_get_parser.add_argument('key', help='配置键')
        
        # config set
        config_set_parser = config_subparsers.add_parser('set', help='✏️ 设置配置值')
        config_set_parser.add_argument('key', help='配置键')
        config_set_parser.add_argument('value', help='配置值')
        
        # config list
        config_list_parser = config_subparsers.add_parser('list', help='📋 列出所有配置')
        config_list_parser.add_argument('--format', choices=['table', 'json', 'yaml'], default='table', help='输出格式')
        
        # config edit
        config_edit_parser = config_subparsers.add_parser('edit', help='✏️ 编辑配置文件')
        config_edit_parser.add_argument('--editor', help='指定编辑器')
        
        # Monitor命令组
        monitor_parser = subparsers.add_parser('monitor', help='📊 监控命令')
        monitor_subparsers = monitor_parser.add_subparsers(dest='monitor_action')
        
        # monitor system
        monitor_sys_parser = monitor_subparsers.add_parser('system', help='💻 系统监控')
        monitor_sys_parser.add_argument('--interval', type=int, default=5, help='刷新间隔（秒）')
        
        # monitor agents
        monitor_agents_parser = monitor_subparsers.add_parser('agents', help='🤖 Agent监控')
        monitor_agents_parser.add_argument('--interval', type=int, default=10, help='刷新间隔（秒）')
        
        # monitor workflows
        monitor_wf_parser = monitor_subparsers.add_parser('workflows', help='⚙️ 工作流监控')
        monitor_wf_parser.add_argument('--interval', type=int, default=15, help='刷新间隔（秒）')
        
        # Tools命令组
        tools_parser = subparsers.add_parser('tools', help='🛠️ 工具命令')
        tools_subparsers = tools_parser.add_subparsers(dest='tools_action')
        
        # tools validate
        tools_validate_parser = tools_subparsers.add_parser('validate', help='✅ 验证配置和设置')
        tools_validate_parser.add_argument('--fix', action='store_true', help='自动修复问题')
        
        # tools benchmark
        tools_bench_parser = tools_subparsers.add_parser('benchmark', help='⚡ 性能基准测试')
        tools_bench_parser.add_argument('--type', choices=['agent', 'workflow', 'system'], default='system', help='测试类型')
        tools_bench_parser.add_argument('--duration', type=int, default=60, help='测试时长（秒）')
        
        # tools export
        tools_export_parser = tools_subparsers.add_parser('export', help='📤 导出数据')
        tools_export_parser.add_argument('--type', choices=['agents', 'workflows', 'teams', 'all'], required=True, help='导出类型')
        tools_export_parser.add_argument('--format', choices=['json', 'yaml', 'csv'], default='json', help='导出格式')
        tools_export_parser.add_argument('--output', help='输出文件路径')
        
        # tools import
        tools_import_parser = tools_subparsers.add_parser('import', help='📥 导入数据')
        tools_import_parser.add_argument('file', help='导入文件路径')
        tools_import_parser.add_argument('--type', choices=['agents', 'workflows', 'teams', 'auto'], default='auto', help='导入类型')
        tools_import_parser.add_argument('--merge', action='store_true', help='合并模式（不覆盖现有数据）')
        
        # Demo命令组 - 增强版
        demo_parser = subparsers.add_parser('demo', help='🎮 演示命令')
        demo_subparsers = demo_parser.add_subparsers(dest='demo_type')
        
        # demo openai
        openai_parser = demo_subparsers.add_parser('openai', help='🤖 OpenAI演示')
        openai_parser.add_argument('--model', default='gpt-4o-mini', help='使用的模型')
        openai_parser.add_argument('--interactive', action='store_true', help='交互模式')
        
        # demo business
        business_parser = demo_subparsers.add_parser('business', help='🏢 业务层演示')
        business_parser.add_argument('--module', choices=['collaboration', 'workflow', 'team', 'all'], default='all', help='演示模块')
        business_parser.add_argument('--verbose', action='store_true', help='详细输出')
        
        # demo orchestration
        orchestration_parser = demo_subparsers.add_parser('orchestration', help='🎭 应用编排演示')
        orchestration_parser.add_argument('--verbose', action='store_true', help='详细输出')
        
        # demo interactive
        interactive_parser = demo_subparsers.add_parser('interactive', help='🎯 交互式演示向导')
        
        # Help命令组
        help_parser = subparsers.add_parser('help', help='❓ 帮助命令')
        help_subparsers = help_parser.add_subparsers(dest='help_topic')
        
        # help commands
        help_cmd_parser = help_subparsers.add_parser('commands', help='📋 命令列表')
        help_cmd_parser.add_argument('--category', help='命令分类')
        
        # help examples
        help_examples_parser = help_subparsers.add_parser('examples', help='💡 使用示例')
        help_examples_parser.add_argument('--command', help='特定命令的示例')
        
        # help docs
        help_docs_parser = help_subparsers.add_parser('docs', help='📚 打开文档')
        help_docs_parser.add_argument('--local', action='store_true', help='打开本地文档')
        
    async def execute_command(self, args: argparse.Namespace) -> int:
        """执行命令"""
        try:
            if args.command == 'agent':
                return await self._handle_agent_commands(args)
            elif args.command == 'workflow':
                return await self._handle_workflow_commands(args)
            elif args.command == 'team':
                return await self._handle_team_commands(args)
            elif args.command == 'project':
                return await self._handle_project_commands(args)
            elif args.command == 'config':
                return await self._handle_config_commands(args)
            elif args.command == 'monitor':
                return await self._handle_monitor_commands(args)
            elif args.command == 'tools':
                return await self._handle_tools_commands(args)
            elif args.command == 'demo':
                return await self._handle_demo_commands(args)
            elif args.command == 'help':
                return await self._handle_help_commands(args)
            else:
                print(f"未知命令: {args.command}")
                return 1
                
        except Exception as e:
            print(f"❌ 命令执行错误: {e}")
            return 1
    
    def _print_table(self, data: List[Dict], title: str = ""):
        """打印表格"""
        if RICH_AVAILABLE and hasattr(self, 'console'):
            table = Table(title=title)
            if data:
                for key in data[0].keys():
                    table.add_column(key, style="cyan")
                for row in data:
                    table.add_row(*[str(v) for v in row.values()])
                self.console.print(table)
        else:
            # 回退到简单文本表格
            if data:
                headers = list(data[0].keys())
                print(f"{'  '.join(headers)}")
                print("-" * 50)
                for row in data:
                    print(f"{'  '.join(str(v) for v in row.values())}")
    
    async def _handle_agent_commands(self, args: argparse.Namespace) -> int:
        """处理Agent命令"""
        if args.agent_action == 'list':
            return await self._agent_list(args)
        elif args.agent_action == 'create':
            return await self._agent_create(args)
        elif args.agent_action == 'chat':
            return await self._agent_chat(args)
        elif args.agent_action == 'info':
            return await self._agent_info(args)
        elif args.agent_action == 'delete':
            return await self._agent_delete(args)
        else:
            print(f"未知Agent操作: {args.agent_action}")
            return 1
    
    async def _agent_list(self, args: argparse.Namespace) -> int:
        """列出Agent"""
        print("📋 Agent列表:")
        print("=" * 50)
        
        # 这里应该从实际的Agent管理器获取数据
        # 目前显示示例数据
        agents = [
            {"名称": "OpenAI Assistant", "类型": "openai", "模型": "gpt-4o-mini", "状态": "ready"},
            {"名称": "Code Expert", "类型": "autogen", "模型": "multi", "状态": "inactive"},
            {"名称": "AutoGen Team", "类型": "autogen", "模型": "multi", "状态": "ready"},
        ]
        
        # 应用过滤器
        if args.filter:
            agents = [a for a in agents if args.filter.lower() in a["名称"].lower() or args.filter.lower() in a["类型"].lower()]
        
        # 应用排序
        if args.sort == 'name':
            agents.sort(key=lambda x: x["名称"])
        elif args.sort == 'type':
            agents.sort(key=lambda x: x["类型"])
        
        if args.format == 'json':
            print(json.dumps(agents, indent=2, ensure_ascii=False))
        elif args.format == 'yaml':
            print(yaml.dump(agents, indent=2, allow_unicode=True))
        else:
            self._print_table(agents, "Agent列表")
        
        return 0
    
    async def _agent_create(self, args: argparse.Namespace) -> int:
        """创建Agent - 真正的Zeus开发体验层实现"""
        print(f"✨ 创建新Agent: {args.name}")
        print("=" * 50)
        
        try:
            # 导入真正的Agent工厂
            from .agent_factory import ZeusAgentFactory, AgentSpec, AgentType, TemplateType
            
            # 创建工厂实例
            factory = ZeusAgentFactory()
            
            # 解析Agent类型
            agent_type_map = {
                'openai': AgentType.OPENAI,
                'autogen': AgentType.AUTOGEN,
                'langgraph': AgentType.LANGGRAPH,
                'fpga_expert': AgentType.FPGA_EXPERT,
                'code_expert': AgentType.CODE_EXPERT,
                'custom': AgentType.CUSTOM
            }
            agent_type = agent_type_map.get(args.type, AgentType.OPENAI)
            
            # 确定模板类型
            if args.template:
                template_map = {
                    'basic': TemplateType.BASIC,
                    'advanced': TemplateType.ADVANCED,
                    'enterprise': TemplateType.ENTERPRISE,
                    'fpga_basic': TemplateType.FPGA_BASIC,
                    'fpga_advanced': TemplateType.FPGA_ADVANCED,
                    'verification_expert': TemplateType.VERIFICATION_EXPERT
                }
                template_type = template_map.get(args.template, TemplateType.BASIC)
            else:
                # 根据Agent类型自动选择模板
                if agent_type == AgentType.FPGA_EXPERT:
                    template_type = TemplateType.FPGA_ADVANCED
                else:
                    template_type = TemplateType.ADVANCED
            
            # 处理能力和知识域
            capabilities = args.capabilities or self._get_default_capabilities(agent_type)
            knowledge_domains = self._get_default_knowledge_domains(agent_type)
            
            # 创建Agent规格
            spec = AgentSpec(
                name=args.name,
                agent_type=agent_type,
                template_type=template_type,
                model=args.model,
                capabilities=capabilities,
                knowledge_domains=knowledge_domains,
                system_message=args.system_message
            )
            
            # 显示创建信息
            print(f"   名称: {spec.name}")
            print(f"   类型: {spec.agent_type.value}")
            print(f"   模板: {spec.template_type.value}")
            print(f"   模型: {spec.model}")
            print(f"   能力: {', '.join(spec.capabilities)}")
            print(f"   知识域: {', '.join(spec.knowledge_domains)}")
            if spec.system_message:
                print(f"   系统消息: {spec.system_message[:50]}...")
            
            # 实际创建Agent
            print(f"\n🔧 正在生成Agent文件...")
            result = await factory.create_agent(spec)
            
            if result.success:
                print(f"\n🎉 Agent创建成功！")
                print(f"   📁 位置: {result.agent_path}")
                print(f"   📋 配置: {result.config_path}")
                print(f"   🐍 主脚本: {result.main_script}")
                print(f"   📄 生成文件: {len(result.generated_files)}个")
                
                print(f"\n🚀 下一步:")
                print(f"   1. cd {result.agent_path}")
                print(f"   2. pip install -r requirements.txt")
                print(f"   3. python {result.main_script.name}")
                print(f"   4. zeus agent chat --name {args.name}")
                
                return 0
            else:
                print(f"❌ 创建失败: {result.error_message}")
                return 1
                
        except ImportError as e:
            print(f"❌ 导入Agent工厂失败: {e}")
            print("请确保Zeus平台核心组件已正确安装")
            return 1
        except Exception as e:
            print(f"❌ 创建Agent失败: {e}")
            import traceback
            traceback.print_exc()
            return 1
    
    def _get_default_capabilities(self, agent_type: 'AgentType') -> List[str]:
        """获取默认能力"""
        from .agent_factory import AgentType
        
        if agent_type == AgentType.FPGA_EXPERT:
            return ["testbench_generation", "timing_analysis", "synthesis_optimization", "debug_assistance"]
        elif agent_type == AgentType.CODE_EXPERT:
            return ["code_generation", "code_review", "bug_detection", "optimization"]
        else:
            return ["chat", "information_retrieval", "analysis"]
    
    def _get_default_knowledge_domains(self, agent_type: 'AgentType') -> List[str]:
        """获取默认知识域"""
        from .agent_factory import AgentType
        
        if agent_type == AgentType.FPGA_EXPERT:
            return ["fpga", "verilog", "verification", "timing"]
        elif agent_type == AgentType.CODE_EXPERT:
            return ["programming", "software_engineering", "algorithms"]
        else:
            return ["general"]
    
    async def _agent_chat(self, args: argparse.Namespace) -> int:
        """与Agent对话"""
        print(f"💬 与Agent对话: {args.name}")
        print("=" * 50)
        
        if args.message:
            # 单次对话
            print(f"用户: {args.message}")
            print(f"Agent: 这是来自{args.name}的回复（模拟）")
            
            if args.save:
                # 保存对话到文件
                with open(args.save, 'a', encoding='utf-8') as f:
                    f.write(f"用户: {args.message}\n")
                    f.write(f"Agent: 这是来自{args.name}的回复（模拟）\n\n")
                print(f"✅ 对话已保存到: {args.save}")
        else:
            # 交互模式
            print("进入交互模式，输入 'quit' 退出")
            while True:
                try:
                    user_input = input("用户: ").strip()
                    if user_input.lower() in ['quit', 'exit', '退出']:
                        break
                    print(f"Agent: 这是来自{args.name}的回复（模拟）")
                except KeyboardInterrupt:
                    break
        
        return 0
    
    async def _agent_info(self, args: argparse.Namespace) -> int:
        """显示Agent详细信息"""
        print(f"ℹ️ 显示Agent详细信息: {args.name}")
        print("=" * 50)
        
        # 这里应该从实际的Agent管理器获取数据
        # 目前显示示例数据
        agent_info = [
            {"属性": "名称", "值": args.name},
            {"属性": "类型", "值": "openai"},
            {"属性": "模型", "值": "gpt-4o-mini"},
            {"属性": "状态", "值": "ready"},
            {"属性": "创建时间", "值": "2023-10-27 10:00:00"},
            {"属性": "能力", "值": "自然语言处理, 代码生成, 信息检索"}
        ]
        
        self._print_table(agent_info, f"Agent: {args.name}")
        return 0
    
    async def _agent_delete(self, args: argparse.Namespace) -> int:
        """删除Agent"""
        print(f"🗑️ 删除Agent: {args.name}")
        print("=" * 50)
        
        if args.force:
            confirm = "y"
        else:
            confirm = input("确定要删除该Agent吗？(y/N): ").lower()
            
        if confirm == "y":
            try:
                # 这里应该从实际的Agent管理器删除
                print(f"✅ 已删除Agent: {args.name}")
                return 0
            except Exception as e:
                print(f"❌ 删除Agent失败: {e}")
                return 1
        else:
            print("👋 取消删除")
            return 0

    # 添加其他命令处理方法的简化版本...
    async def _handle_workflow_commands(self, args: argparse.Namespace) -> int:
        """处理工作流命令"""
        print(f"⚙️ 工作流命令: {args.workflow_action}")
        return 0

    async def _handle_team_commands(self, args: argparse.Namespace) -> int:
        """处理团队命令"""
        print(f"👥 团队命令: {args.team_action}")
        return 0

    async def _handle_project_commands(self, args: argparse.Namespace) -> int:
        """处理项目命令"""
        print(f"🏗️ 项目命令: {args.project_action}")
        return 0

    async def _handle_config_commands(self, args: argparse.Namespace) -> int:
        """处理配置命令"""
        print(f"⚙️ 配置命令: {args.config_action}")
        return 0

    async def _handle_monitor_commands(self, args: argparse.Namespace) -> int:
        """处理监控命令"""
        print(f"📊 监控命令: {args.monitor_action}")
        return 0

    async def _handle_tools_commands(self, args: argparse.Namespace) -> int:
        """处理工具命令"""
        print(f"🛠️ 工具命令: {args.tools_action}")
        return 0

    async def _handle_demo_commands(self, args: argparse.Namespace) -> int:
        """处理演示命令"""
        if args.demo_type == 'openai':
            return await self._run_openai_demo(args)
        elif args.demo_type == 'business':
            return await self._run_business_demo(args)
        elif args.demo_type == 'orchestration':
            return await self._run_orchestration_demo(args)
        elif args.demo_type == 'interactive':
            return await self._run_interactive_demo()
        else:
            print(f"未知演示类型: {args.demo_type}")
            return 1

    async def _run_openai_demo(self, args: argparse.Namespace) -> int:
        """运行OpenAI演示"""
        print("🤖 运行OpenAI演示")
        print("=" * 50)
        
        try:
            # 运行实际的OpenAI演示
            result = subprocess.run([
                sys.executable, "examples/openai_demo.py"
            ], cwd=project_root, capture_output=False)
            return result.returncode
        except Exception as e:
            print(f"❌ OpenAI演示执行失败: {e}")
            return 1

    async def _run_business_demo(self, args: argparse.Namespace) -> int:
        """运行业务层演示"""
        print("🏢 运行业务层演示")
        print("=" * 50)
        
        try:
            # 运行实际的业务层演示
            env = os.environ.copy()
            env['PYTHONPATH'] = str(project_root)
            result = subprocess.run([
                sys.executable, "examples/business_layer_demo.py"
            ], cwd=project_root, capture_output=False, env=env)
            return result.returncode
        except Exception as e:
            print(f"❌ 业务层演示执行失败: {e}")
            return 1

    async def _run_orchestration_demo(self, args: argparse.Namespace) -> int:
        """运行应用编排演示"""
        print("🎭 运行应用编排演示")
        print("=" * 50)
        
        try:
            # 运行实际的应用编排演示
            env = os.environ.copy()
            env['PYTHONPATH'] = str(project_root)
            result = subprocess.run([
                sys.executable, "examples/application_orchestration_demo.py"
            ], cwd=project_root, capture_output=False, env=env)
            return result.returncode
        except Exception as e:
            print(f"❌ 应用编排演示执行失败: {e}")
            return 1

    async def _run_interactive_demo(self) -> int:
        """运行交互式演示向导"""
        print("🎯 交互式演示向导")
        print("=" * 50)
        
        print("欢迎使用ADC交互式演示向导！")
        print("请选择您想要演示的功能：")
        print("1. OpenAI适配器演示")
        print("2. 业务能力层演示") 
        print("3. 应用编排层演示")
        print("4. 退出")
        
        while True:
            try:
                choice = input("请选择 (1-4): ").strip()
                if choice == '1':
                    return await self._run_openai_demo(argparse.Namespace(model='gpt-4o-mini', interactive=True))
                elif choice == '2':
                    return await self._run_business_demo(argparse.Namespace(module='all', verbose=True))
                elif choice == '3':
                    return await self._run_orchestration_demo(argparse.Namespace(verbose=True))
                elif choice == '4':
                    print("👋 再见！")
                    return 0
                else:
                    print("❌ 无效选择，请重试")
            except KeyboardInterrupt:
                print("\n👋 再见！")
                return 0

    async def _handle_help_commands(self, args: argparse.Namespace) -> int:
        """处理帮助命令"""
        if args.help_topic == 'commands':
            return await self._help_commands(args)
        elif args.help_topic == 'examples':
            return await self._help_examples(args)
        elif args.help_topic == 'docs':
            return await self._help_docs(args)
        else:
            print("❓ 帮助:")
            print("=" * 50)
            print("请使用 'help commands' 查看所有命令。")
            print("使用 'help examples <command>' 查看特定命令的示例。")
            print("使用 'help docs' 打开文档。")
            return 0

    async def _help_commands(self, args: argparse.Namespace) -> int:
        """显示所有命令"""
        print("📋 所有命令:")
        print("=" * 50)
        
        commands = [
            {"分类": "Agent管理", "命令": "list, create, chat, info, delete"},
            {"分类": "工作流管理", "命令": "list, create, run, status, logs"},
            {"分类": "团队管理", "命令": "list, create, collaborate, performance"},
            {"分类": "项目管理", "命令": "init, list, status, build, deploy"},
            {"分类": "配置管理", "命令": "get, set, list, edit"},
            {"分类": "监控", "命令": "system, agents, workflows"},
            {"分类": "工具", "命令": "validate, benchmark, export, import"},
            {"分类": "演示", "命令": "openai, business, orchestration, interactive"},
            {"分类": "帮助", "命令": "commands, examples, docs"}
        ]
        
        self._print_table(commands, "命令分类")
        return 0

    async def _help_examples(self, args: argparse.Namespace) -> int:
        """显示特定命令的示例"""
        print(f"💡 使用示例: {args.command}")
        print("=" * 50)
        
        if args.command == 'agent create':
            print("✨ 创建新Agent示例:")
            print("  adc agent create --name MyAgent --type openai --model gpt-4o-mini")
            print("  adc agent create --name MyAgent --interactive")
        elif args.command == 'demo business':
            print("🏢 业务层演示示例:")
            print("  adc demo business --module all --verbose")
        else:
            print(f"未找到命令 '{args.command}' 的示例。")
        
        return 0

    async def _help_docs(self, args: argparse.Namespace) -> int:
        """打开文档"""
        print("📚 打开文档:")
        print("=" * 50)
        
        if args.local:
            try:
                readme_path = project_root / "README.md"
                if readme_path.exists():
                    webbrowser.open(f"file://{readme_path}")
                    print(f"✅ 已打开本地文档: {readme_path}")
                else:
                    print(f"❌ 本地文档不存在: {readme_path}")
                    return 1
            except Exception as e:
                print(f"❌ 打开本地文档失败: {e}")
                return 1
        else:
            print("✅ 文档链接: https://github.com/fpga1988/zeus")
        
        return 0 


"""
认知架构层 - Cognitive Architecture Layer
提供Agent的认知能力：感知、推理、记忆、学习、通信
"""

# 认知Agent
from .cognitive_agent import (
    CognitiveAgent, CognitiveState, AgentIdentity, CognitiveMetrics,
    CognitiveMessageHandler
)

# 感知模块
from .perception import (
    PerceptionEngine, TextPerceptor, StructuredDataPerceptor,
    BasePerceptor, PerceptionResult, TextPerceptionResult,
    PerceptionType, SentimentType
)

# 推理模块
from .reasoning import (
    ReasoningEngine, LogicalReasoner, CausalReasoner,
    AnalogicalReasoner, InductiveReasoner, BaseReasoner,
    ReasoningResult, ReasoningStep, ReasoningType, ConfidenceLevel
)

# 记忆模块
from .memory import (
    MemorySystem, WorkingMemory, EpisodicMemory,
    SemanticMemory, ProceduralMemory, MemoryConsolidator,
    MemoryRetriever, ForgettingMechanism, MemoryType
)

# 学习模块
from .learning import (
    LearningModule, SupervisedLearning, UnsupervisedLearning,
    ReinforcementLearning, MetaLearning, ExperienceBuffer,
    PatternRecognizer, SkillAcquisition, LearningType, Experience, Skill
)

# 通信模块
from .communication import (
    CommunicationManager, MessageHandler, Message, MessageType,
    TeamProtocol, CommunicationChannel
)

# 层间通信管理器
from .communication_manager import (
    CognitiveCommunicationManager
)

__all__ = [
    # 认知Agent
    "CognitiveAgent", "CognitiveState", "AgentIdentity", "CognitiveMetrics",
    "CognitiveMessageHandler",
    
    # 感知模块
    "PerceptionEngine", "TextPerceptor", "StructuredDataPerceptor",
    "BasePerceptor", "PerceptionResult", "TextPerceptionResult",
    "PerceptionType", "SentimentType",
    
    # 推理模块
    "ReasoningEngine", "LogicalReasoner", "CausalReasoner",
    "AnalogicalReasoner", "InductiveReasoner", "BaseReasoner",
    "ReasoningResult", "ReasoningStep", "ReasoningType", "ConfidenceLevel",
    
    # 记忆模块
    "MemorySystem", "WorkingMemory", "EpisodicMemory",
    "SemanticMemory", "ProceduralMemory", "MemoryConsolidator",
    "MemoryRetriever", "ForgettingMechanism", "MemoryType",
    
    # 学习模块
    "LearningModule", "SupervisedLearning", "UnsupervisedLearning",
    "ReinforcementLearning", "MetaLearning", "ExperienceBuffer",
    "PatternRecognizer", "SkillAcquisition", "LearningType", "Experience", "Skill",
    
    # 通信模块
    "CommunicationManager", "MessageHandler", "Message", "MessageType",
    "TeamProtocol", "CommunicationChannel",
    
    # 层间通信管理器
    "CognitiveCommunicationManager"
]

__version__ = "1.0.0"
__author__ = "Agent Development Center Cognitive Team" 


"""
智能日志系统
支持结构化日志、上下文追踪、Agent状态记录和性能监控
"""

import json
import logging
import time
from datetime import datetime
from typing import Dict, Any, Optional, List
from enum import Enum
from dataclasses import dataclass, asdict
from pathlib import Path
import threading
from contextlib import contextmanager


class LogLevel(Enum):
    """日志级别枚举"""
    DEBUG = "DEBUG"
    INFO = "INFO"
    WARNING = "WARNING"
    ERROR = "ERROR"
    CRITICAL = "CRITICAL"


class LogCategory(Enum):
    """日志分类枚举"""
    AGENT = "agent"
    ADAPTER = "adapter"
    TASK = "task"
    SYSTEM = "system"
    PERFORMANCE = "performance"
    SECURITY = "security"
    USER = "user"


@dataclass
class LogContext:
    """日志上下文信息"""
    session_id: Optional[str] = None
    agent_id: Optional[str] = None
    task_id: Optional[str] = None
    adapter_name: Optional[str] = None
    user_id: Optional[str] = None
    trace_id: Optional[str] = None
    
    def to_dict(self) -> Dict[str, Any]:
        """转换为字典"""
        return {k: v for k, v in asdict(self).items() if v is not None}


@dataclass
class LogEntry:
    """结构化日志条目"""
    timestamp: datetime
    level: LogLevel
    category: LogCategory
    message: str
    context: LogContext
    metadata: Dict[str, Any]
    duration_ms: Optional[float] = None
    
    def to_dict(self) -> Dict[str, Any]:
        """转换为字典格式"""
        return {
            "timestamp": self.timestamp.isoformat(),
            "level": self.level.value,
            "category": self.category.value,
            "message": self.message,
            "context": self.context.to_dict(),
            "metadata": self.metadata,
            "duration_ms": self.duration_ms
        }


class AgentLogger:
    """Agent智能日志记录器"""
    
    def __init__(self, 
                 name: str = "zeus",
                 log_file: Optional[str] = None,
                 max_file_size: int = 10 * 1024 * 1024,  # 10MB
                 backup_count: int = 5,
                 enable_console: bool = True,
                 enable_structured: bool = True):
        
        self.name = name
        self.enable_structured = enable_structured
        self._context_stack = threading.local()
        
        # 创建Python标准logger
        self.logger = logging.getLogger(name)
        self.logger.setLevel(logging.DEBUG)
        
        # 清除现有handlers
        for handler in self.logger.handlers[:]:
            self.logger.removeHandler(handler)
        
        # 控制台输出
        if enable_console:
            console_handler = logging.StreamHandler()
            console_formatter = self._create_formatter(structured=False)
            console_handler.setFormatter(console_formatter)
            self.logger.addHandler(console_handler)
        
        # 文件输出
        if log_file:
            from logging.handlers import RotatingFileHandler
            
            # 确保日志目录存在
            log_path = Path(log_file)
            log_path.parent.mkdir(parents=True, exist_ok=True)
            
            file_handler = RotatingFileHandler(
                log_file, 
                maxBytes=max_file_size,
                backupCount=backup_count,
                encoding='utf-8'
            )
            
            file_formatter = self._create_formatter(structured=enable_structured)
            file_handler.setFormatter(file_formatter)
            self.logger.addHandler(file_handler)
        
        # 性能统计
        self.performance_stats = {
            'total_logs': 0,
            'logs_by_level': {},
            'logs_by_category': {},
            'start_time': datetime.now()
        }
    
    def _create_formatter(self, structured: bool = True) -> logging.Formatter:
        """创建日志格式化器"""
        if structured:
            return StructuredFormatter()
        else:
            return logging.Formatter(
                '%(asctime)s - %(name)s - %(levelname)s - %(message)s',
                datefmt='%Y-%m-%d %H:%M:%S'
            )
    
    def _get_context_stack(self) -> List[LogContext]:
        """获取当前线程的上下文栈"""
        if not hasattr(self._context_stack, 'stack'):
            self._context_stack.stack = []
        return self._context_stack.stack
    
    def _merge_context(self) -> LogContext:
        """合并上下文栈中的所有上下文"""
        stack = self._get_context_stack()
        if not stack:
            return LogContext()
        
        # 从底层到顶层合并上下文
        merged = LogContext()
        for context in stack:
            for field, value in asdict(context).items():
                if value is not None:
                    setattr(merged, field, value)
        
        return merged
    
    @contextmanager
    def context(self, **kwargs):
        """上下文管理器，用于设置日志上下文"""
        context = LogContext(**kwargs)
        stack = self._get_context_stack()
        stack.append(context)
        try:
            yield
        finally:
            stack.pop()
    
    @contextmanager
    def timer(self, message: str, category: LogCategory = LogCategory.PERFORMANCE, **kwargs):
        """计时上下文管理器"""
        start_time = time.time()
        try:
            yield
        finally:
            duration_ms = (time.time() - start_time) * 1000
            self.info(
                message=f"{message} (耗时: {duration_ms:.2f}ms)",
                category=category,
                duration_ms=duration_ms,
                **kwargs
            )
    
    def _log(self, 
             level: LogLevel,
             message: str,
             category: LogCategory = LogCategory.SYSTEM,
             duration_ms: Optional[float] = None,
             **metadata):
        """内部日志记录方法"""
        
        # 合并上下文
        context = self._merge_context()
        
        # 创建日志条目
        entry = LogEntry(
            timestamp=datetime.now(),
            level=level,
            category=category,
            message=message,
            context=context,
            metadata=metadata,
            duration_ms=duration_ms
        )
        
        # 更新统计信息
        self.performance_stats['total_logs'] += 1
        self.performance_stats['logs_by_level'][level.value] = \
            self.performance_stats['logs_by_level'].get(level.value, 0) + 1
        self.performance_stats['logs_by_category'][category.value] = \
            self.performance_stats['logs_by_category'].get(category.value, 0) + 1
        
        # 输出日志
        if self.enable_structured:
            # 结构化日志输出
            log_data = entry.to_dict()
            self.logger.log(
                getattr(logging, level.value),
                json.dumps(log_data, ensure_ascii=False, default=str)
            )
        else:
            # 标准格式输出
            context_str = ""
            if context.agent_id:
                context_str += f"[Agent:{context.agent_id}]"
            if context.task_id:
                context_str += f"[Task:{context.task_id}]"
            if context.session_id:
                context_str += f"[Session:{context.session_id}]"
            
            full_message = f"{context_str} [{category.value.upper()}] {message}"
            if metadata:
                full_message += f" | {metadata}"
            
            self.logger.log(getattr(logging, level.value), full_message)
    
    def debug(self, message: str, category: LogCategory = LogCategory.SYSTEM, **metadata):
        """调试日志"""
        self._log(LogLevel.DEBUG, message, category, **metadata)
    
    def info(self, message: str, category: LogCategory = LogCategory.SYSTEM, **metadata):
        """信息日志"""
        self._log(LogLevel.INFO, message, category, **metadata)
    
    def warning(self, message: str, category: LogCategory = LogCategory.SYSTEM, **metadata):
        """警告日志"""
        self._log(LogLevel.WARNING, message, category, **metadata)
    
    def error(self, message: str, category: LogCategory = LogCategory.SYSTEM, **metadata):
        """错误日志"""
        self._log(LogLevel.ERROR, message, category, **metadata)
    
    def critical(self, message: str, category: LogCategory = LogCategory.SYSTEM, **metadata):
        """严重错误日志"""
        self._log(LogLevel.CRITICAL, message, category, **metadata)
    
    def agent_action(self, agent_id: str, action: str, **metadata):
        """记录Agent行为"""
        with self.context(agent_id=agent_id):
            self.info(f"Agent行为: {action}", LogCategory.AGENT, **metadata)
    
    def task_start(self, task_id: str, task_type: str, **metadata):
        """记录任务开始"""
        with self.context(task_id=task_id):
            self.info(f"任务开始: {task_type}", LogCategory.TASK, **metadata)
    
    def task_complete(self, task_id: str, result: str, duration_ms: float, **metadata):
        """记录任务完成"""
        with self.context(task_id=task_id):
            self.info(
                f"任务完成: {result}",
                LogCategory.TASK,
                duration_ms=duration_ms,
                **metadata
            )
    
    def adapter_call(self, adapter_name: str, method: str, **metadata):
        """记录适配器调用"""
        with self.context(adapter_name=adapter_name):
            self.info(f"适配器调用: {method}", LogCategory.ADAPTER, **metadata)
    
    def security_event(self, event_type: str, severity: str, **metadata):
        """记录安全事件"""
        level = LogLevel.WARNING if severity == 'medium' else LogLevel.ERROR
        self._log(level, f"安全事件: {event_type}", LogCategory.SECURITY, **metadata)
    
    def get_stats(self) -> Dict[str, Any]:
        """获取日志统计信息"""
        uptime = (datetime.now() - self.performance_stats['start_time']).total_seconds()
        return {
            **self.performance_stats,
            'uptime_seconds': uptime,
            'logs_per_second': self.performance_stats['total_logs'] / max(uptime, 1)
        }


class StructuredFormatter(logging.Formatter):
    """结构化日志格式化器"""
    
    def format(self, record):
        """格式化日志记录"""
        try:
            # 如果消息已经是JSON格式，直接返回
            json.loads(record.getMessage())
            return record.getMessage()
        except (json.JSONDecodeError, ValueError):
            # 如果不是JSON，创建结构化格式
            log_entry = {
                "timestamp": datetime.fromtimestamp(record.created).isoformat(),
                "level": record.levelname,
                "logger": record.name,
                "message": record.getMessage(),
                "module": record.module,
                "function": record.funcName,
                "line": record.lineno
            }
            
            # 添加异常信息
            if record.exc_info:
                log_entry["exception"] = self.formatException(record.exc_info)
            
            return json.dumps(log_entry, ensure_ascii=False, default=str)


# 全局日志实例
_global_logger = None


def get_logger(name: str = "zeus", **kwargs) -> AgentLogger:
    """获取全局日志实例"""
    global _global_logger
    
    if _global_logger is None:
        # 默认配置
        default_config = {
            'log_file': 'logs/zeus.log',
            'enable_console': True,
            'enable_structured': True
        }
        default_config.update(kwargs)
        
        _global_logger = AgentLogger(name, **default_config)
    
    return _global_logger


def configure_logging(config: Dict[str, Any]):
    """配置全局日志"""
    global _global_logger
    _global_logger = AgentLogger(**config)


# 便捷函数
def debug(message: str, category: LogCategory = LogCategory.SYSTEM, **metadata):
    """全局调试日志"""
    get_logger().debug(message, category, **metadata)


def info(message: str, category: LogCategory = LogCategory.SYSTEM, **metadata):
    """全局信息日志"""
    get_logger().info(message, category, **metadata)


def warning(message: str, category: LogCategory = LogCategory.SYSTEM, **metadata):
    """全局警告日志"""
    get_logger().warning(message, category, **metadata)


def error(message: str, category: LogCategory = LogCategory.SYSTEM, **metadata):
    """全局错误日志"""
    get_logger().error(message, category, **metadata)


def critical(message: str, category: LogCategory = LogCategory.SYSTEM, **metadata):
    """全局严重错误日志"""
    get_logger().critical(message, category, **metadata) 


"""
AutoGen v0.4 Adapter Comprehensive Tests
Complete test suite for the new AutoGen v0.4 (autogen-agentchat) adapter
"""

import pytest
import asyncio
import os
import tempfile
import shutil
from unittest.mock import Mock, patch, AsyncMock
from datetime import datetime

# Test target imports
from layers.adapter.autogen.adapter import (
    AutoGenV04Adapter, 
    create_autogen_v04_adapter,
    AUTOGEN_V04_AVAILABLE
)
from layers.framework.abstractions.task import (
    UniversalTask, 
    TaskType, 
    TaskPriority, 
    TaskRequirements
)
from layers.framework.abstractions.result import ResultStatus
from layers.framework.abstractions.context import UniversalContext
from layers.adapter.base import (
    AdapterError, 
    AdapterInitializationError, 
    AdapterExecutionError
)


@pytest.mark.autogen
class TestAutoGenV04Adapter:
    """Comprehensive test suite for AutoGen v0.4 adapter"""
    
    @pytest.fixture(autouse=True)
    def setup_method(self):
        """Test setup"""
        self.test_api_key = os.getenv("OPENAI_API_KEY_TEST", "test-key-12345")
        self.real_api_key = os.getenv("OPENAI_API_KEY")
        self.work_dir = tempfile.mkdtemp()
        
        yield
        
        # Cleanup
        if os.path.exists(self.work_dir):
            shutil.rmtree(self.work_dir)
    
    def test_adapter_availability(self):
        """Test AutoGen v0.4 availability detection"""
        if AUTOGEN_V04_AVAILABLE:
            # Test successful initialization
            adapter = AutoGenV04Adapter("test_adapter")
            assert adapter.name == "test_adapter"
            assert hasattr(adapter, 'agents')
            assert hasattr(adapter, 'llm_backends')
        else:
            # Test error when not available
            with pytest.raises(AdapterInitializationError):
                AutoGenV04Adapter("test_adapter")
    
    def test_adapter_initialization(self):
        """Test adapter initialization with various configs"""
        if not AUTOGEN_V04_AVAILABLE:
            pytest.skip("AutoGen v0.4 not available")
        
        # Test minimal initialization
        adapter1 = AutoGenV04Adapter("minimal")
        assert adapter1.name == "minimal"
        assert adapter1.default_model == "gpt-3.5-turbo"
        
        # Test full configuration
        config = {
            "default_model": "gpt-4",
            "default_temperature": 0.8,
            "work_dir": self.work_dir
        }
        adapter2 = AutoGenV04Adapter("configured", config)
        assert adapter2.default_model == "gpt-4"
        assert adapter2.default_temperature == 0.8
        assert adapter2.work_dir == self.work_dir
    
    def test_adapter_info(self):
        """Test adapter info provides complete information"""
        if not AUTOGEN_V04_AVAILABLE:
            pytest.skip("AutoGen v0.4 not available")
        
        adapter = AutoGenV04Adapter("info_test")
        info = adapter.get_info()
        
        assert info.name == "info_test"
        assert info.version == "0.4.0"
        assert "AutoGen v0.4" in info.description
        assert len(info.capabilities) > 0
        assert len(info.supported_models) > 0
    
    def test_llm_backend_creation(self):
        """Test LLM backend creation and validation"""
        if not AUTOGEN_V04_AVAILABLE:
            pytest.skip("AutoGen v0.4 not available")
        
        adapter = AutoGenV04Adapter("backend_test")
        
        # Test OpenAI backend creation
        backend_id = adapter.create_llm_backend(
            backend_id="openai_backend",
            backend_type="openai",
            api_key=self.test_api_key,
            model="gpt-3.5-turbo"
        )
        assert backend_id == "openai_backend"
        assert backend_id in adapter.llm_backends
        
        # Test DeepSeek backend creation
        backend_id = adapter.create_llm_backend(
            backend_id="deepseek_backend",
            backend_type="deepseek",
            api_key=self.test_api_key,
            model="deepseek-chat"
        )
        assert backend_id == "deepseek_backend"
        assert backend_id in adapter.llm_backends
        
        # Test invalid backend type
        with pytest.raises(AdapterError):
            adapter.create_llm_backend(
                backend_id="invalid",
                backend_type="invalid_type",
                api_key=self.test_api_key
            )
        
        # Test missing API key
        with pytest.raises(AdapterError):
            adapter.create_llm_backend(
                backend_id="no_key",
                backend_type="openai"
            )
    
    def test_assistant_agent_creation(self):
        """Test AssistantAgent creation"""
        if not AUTOGEN_V04_AVAILABLE:
            pytest.skip("AutoGen v0.4 not available")
        
        adapter = AutoGenV04Adapter("agent_test")
        
        # Create LLM backend first
        backend_id = adapter.create_llm_backend(
            backend_id="test_backend",
            backend_type="openai",
            api_key=self.test_api_key,
            model="gpt-3.5-turbo"
        )
        
        # Create assistant agent
        agent_id = adapter.create_assistant_agent(
            agent_id="test_assistant",
            name="test_assistant",
            system_message="You are a helpful assistant",
            backend_id=backend_id
        )
        
        assert agent_id == "test_assistant"
        assert agent_id in adapter.agents
        
        # Test with non-existent backend
        with pytest.raises(AdapterError):
            adapter.create_assistant_agent(
                "invalid_agent", "Invalid", "Test", "non_existent_backend"
            )
    
    def test_code_executor_agent_creation(self):
        """Test CodeExecutorAgent creation"""
        if not AUTOGEN_V04_AVAILABLE:
            pytest.skip("AutoGen v0.4 not available")
        
        adapter = AutoGenV04Adapter("executor_test")
        
        # Create code executor agent
        agent_id = adapter.create_code_executor_agent(
            agent_id="test_executor",
            name="test_executor",
            work_dir=self.work_dir
        )
        
        assert agent_id == "test_executor"
        assert agent_id in adapter.code_executors
    
    def test_user_proxy_agent_creation(self):
        """Test UserProxyAgent creation"""
        if not AUTOGEN_V04_AVAILABLE:
            pytest.skip("AutoGen v0.4 not available")
        
        adapter = AutoGenV04Adapter("proxy_test")
        
        # Create user proxy agent
        agent_id = adapter.create_user_proxy_agent(
            agent_id="test_proxy",
            name="test_proxy"
        )
        
        assert agent_id == "test_proxy"
        assert agent_id in adapter.user_proxies
    
    def test_team_creation(self):
        """Test team creation with various agent types"""
        if not AUTOGEN_V04_AVAILABLE:
            pytest.skip("AutoGen v0.4 not available")
        
        adapter = AutoGenV04Adapter("team_test")
        
        # Create LLM backend and agents
        backend_id = adapter.create_llm_backend(
            backend_id="client",
            backend_type="openai",
            api_key=self.test_api_key,
            model="gpt-3.5-turbo"
        )
        
        assistant_id = adapter.create_assistant_agent(
            "assistant", "assistant", "Test", backend_id
        )
        executor_id = adapter.create_code_executor_agent("executor", "executor")
        proxy_id = adapter.create_user_proxy_agent("proxy", "proxy")
        
        # Create team
        team_id = adapter._create_team_internal(
            team_id="test_team",
            agent_ids=[assistant_id, executor_id],
            max_turns=5,
            termination_keywords=["TERMINATE"]
        )
        
        assert team_id == "test_team"
        assert team_id in adapter.teams
        
        # Test team with non-existent agent
        with pytest.raises(AdapterError):
            adapter._create_team_internal("invalid_team", ["non_existent_agent"])
    
    @pytest.mark.asyncio
    async def test_chat_task_execution(self):
        """Test chat task execution with different LLM backends"""
        if not AUTOGEN_V04_AVAILABLE:
            pytest.skip("AutoGen v0.4 not available")
        
        adapter = AutoGenV04Adapter("chat_test")
        
        # Test with OpenAI backend
        with patch('layers.adapter.autogen.llm_backend.OpenAIBackend') as mock_backend_class:
            with patch('layers.adapter.autogen.adapter.AssistantAgent') as mock_agent_class:
                # Setup mocks
                mock_backend = Mock()
                mock_backend.chat_completion = AsyncMock(return_value={
                    "choices": [{"message": {"content": "Hello! How can I help you?"}}]
                })
                mock_backend_class.return_value = mock_backend
                
                mock_agent = Mock()
                mock_response = Mock()
                mock_response.chat_message.content = "Hello! How can I help you?"
                mock_agent.on_messages = AsyncMock(return_value=mock_response)
                mock_agent.name = "Test Assistant"
                mock_agent_class.return_value = mock_agent
                
                # Create backend and agent
                backend_id = adapter.create_llm_backend(
                    backend_id="openai_test",
                    backend_type="openai",
                    api_key=self.test_api_key
                )
                agent_id = adapter.create_assistant_agent(
                    "openai_agent",
                    "OpenAI Assistant",
                    "Test",
                    backend_id
                )
                
                # Execute task
                task = UniversalTask(
                    content="Hello, how are you?",
                    task_type=TaskType.CONVERSATION,
                    priority=TaskPriority.NORMAL,
                    requirements=TaskRequirements(),
                    context={},
                    task_id="openai_chat_test"
                )
                
                context = UniversalContext({'user_id': 'test_user'})
                result = await adapter.execute_task(task, context)
                
                assert result.status == ResultStatus.SUCCESS
                assert "reply" in result.content
                assert result.content["agent_id"] == agent_id
        
        # Clear agents for next test
        adapter.agents.clear()
        
        # Test with DeepSeek backend
        with patch('layers.adapter.autogen.llm_backend.DeepSeekBackend') as mock_backend_class:
            with patch('layers.adapter.autogen.adapter.AssistantAgent') as mock_agent_class:
                # Setup mocks
                mock_backend = Mock()
                mock_backend.chat_completion = AsyncMock(return_value={
                    "choices": [{"message": {"content": "Hello from DeepSeek!"}}]
                })
                mock_backend_class.return_value = mock_backend
                
                mock_agent = Mock()
                mock_response = Mock()
                mock_response.chat_message.content = "Hello from DeepSeek!"
                mock_agent.on_messages = AsyncMock(return_value=mock_response)
                mock_agent.name = "Test Assistant"
                mock_agent_class.return_value = mock_agent
                
                # Create backend and agent
                backend_id = adapter.create_llm_backend(
                    backend_id="deepseek_test",
                    backend_type="deepseek",
                    api_key=self.test_api_key
                )
                agent_id = adapter.create_assistant_agent(
                    "deepseek_agent",
                    "deepseek_assistant",
                    "Test",
                    backend_id
                )
                
                # Execute task
                task = UniversalTask(
                    content="Hello, how are you?",
                    task_type=TaskType.CONVERSATION,
                    priority=TaskPriority.NORMAL,
                    requirements=TaskRequirements(),
                    context={},
                    task_id="deepseek_chat_test"
                )
                
                context = UniversalContext({'user_id': 'test_user'})
                result = await adapter.execute_task(task, context)
                
                assert result.status == ResultStatus.SUCCESS
                assert "reply" in result.content
                assert result.content["agent_id"] == agent_id
    
    @pytest.mark.asyncio
    async def test_code_task_execution(self):
        """Test code generation task execution"""
        if not AUTOGEN_V04_AVAILABLE:
            pytest.skip("AutoGen v0.4 not available")
        
        adapter = AutoGenV04Adapter("code_test")
        
        # Mock components
        with patch('layers.adapter.autogen.adapter.create_llm_backend'):
            with patch('layers.adapter.autogen.adapter.AssistantAgent'):
                with patch('layers.adapter.autogen.adapter.CodeExecutorAgent'):
                    with patch('layers.adapter.autogen.adapter.RoundRobinGroupChat') as mock_team:
                        # Setup mocks
                        mock_team_instance = Mock()
                        mock_result = Mock()
                        mock_result.messages = ["msg1", "msg2"]
                        mock_team_instance.run = AsyncMock(return_value=mock_result)
                        mock_team.return_value = mock_team_instance
                        
                        # Create components
                        backend_id = adapter.create_llm_backend(
                            backend_id="client",
                            backend_type="openai",
                            api_key=self.test_api_key,
                            model="gpt-3.5-turbo"
                        )
                        agent_id = adapter.create_assistant_agent("agent", "agent", "Test", backend_id)
                        
                        # Execute task
                        task = UniversalTask(
                            content="Write a Python function to calculate factorial",
                            task_type=TaskType.CODE_GENERATION,
                            priority=TaskPriority.NORMAL,
                            requirements=TaskRequirements(),
                            context={},
                            task_id="code_test"
                        )
                        
                        context = UniversalContext({'user_id': 'test_user'})
                        result = await adapter.execute_task(task, context)
                        
                        assert result.status == ResultStatus.SUCCESS
                        assert "code_result" in result.content
    
    @pytest.mark.asyncio
    async def test_collaboration_task_execution(self):
        """Test collaboration task execution"""
        if not AUTOGEN_V04_AVAILABLE:
            pytest.skip("AutoGen v0.4 not available")
        
        adapter = AutoGenV04Adapter("collab_test")
        
        # Mock components
        with patch('layers.adapter.autogen.adapter.create_llm_backend'):
            with patch('layers.adapter.autogen.adapter.AssistantAgent') as mock_agent:
                with patch('layers.adapter.autogen.adapter.RoundRobinGroupChat') as mock_team:
                    # Setup mocks
                    mock_agent_instance = Mock()
                    mock_agent_instance.name = "Test Agent"
                    mock_agent.return_value = mock_agent_instance
                    
                    mock_team_instance = Mock()
                    mock_team_instance.participants = [mock_agent_instance]
                    mock_result = Mock()
                    mock_result.messages = ["msg1", "msg2", "msg3"]
                    mock_team_instance.run = AsyncMock(return_value=mock_result)
                    mock_team.return_value = mock_team_instance
                    
                    # Create components
                    backend_id = adapter.create_llm_backend(
                        backend_id="client",
                        backend_type="openai",
                        api_key=self.test_api_key,
                        model="gpt-3.5-turbo"
                    )
                    agent_id = adapter.create_assistant_agent("agent", "agent", "Test", backend_id)
                    team_id = adapter._create_team_internal("team", [agent_id])
                    
                    # Execute task
                    task = UniversalTask(
                        content="Let's collaborate on solving this problem",
                        task_type=TaskType.TOOL_EXECUTION,
                        priority=TaskPriority.HIGH,
                        requirements=TaskRequirements(),
                        context={},
                        task_id="collab_test"
                    )
                    
                    context = UniversalContext({'user_id': 'test_user'})
                    result = await adapter.execute_task(task, context)
                    
                    assert result.status == ResultStatus.SUCCESS
                    assert "participants" in result.content
                    assert result.content["team_id"] == team_id
    
    @pytest.mark.asyncio
    async def test_task_execution_errors(self):
        """Test task execution error handling"""
        if not AUTOGEN_V04_AVAILABLE:
            pytest.skip("AutoGen v0.4 not available")
        
        adapter = AutoGenV04Adapter("error_test")
        
        # Test unsupported task type
        task = UniversalTask(
            content="Test task",
            task_type="UNSUPPORTED_TYPE",
            priority=TaskPriority.NORMAL,
            requirements=TaskRequirements(),
            context={},
            task_id="error_test"
        )
        
        context = UniversalContext({'user_id': 'test_user'})
        result = await adapter.execute_task(task, context)
        
        assert result.status == ResultStatus.FAILURE
        assert "error" in result.content
    
    def test_agent_status_retrieval(self):
        """Test agent status retrieval"""
        if not AUTOGEN_V04_AVAILABLE:
            pytest.skip("AutoGen v0.4 not available")
        
        adapter = AutoGenV04Adapter("status_test")
        
        # Test non-existent agent
        status = adapter.get_agent_status("non_existent")
        assert status["status"] == "not_found"
        
        # Create agents and test status
        with patch('layers.adapter.autogen.adapter.create_llm_backend'):
            with patch('layers.adapter.autogen.adapter.AssistantAgent') as mock_agent:
                mock_agent_instance = Mock()
                mock_agent_instance.name = "Test Agent"
                mock_agent.return_value = mock_agent_instance
                
                backend_id = adapter.create_llm_backend(
                    backend_id="client",
                    backend_type="openai",
                    api_key=self.test_api_key,
                    model="gpt-3.5-turbo"
                )
                agent_id = adapter.create_assistant_agent("agent", "agent", "Test", backend_id)
                
                status = adapter.get_agent_status(agent_id)
                assert status["status"] == "active"
                assert status["agent_type"] == "assistant"
                assert status["agent_id"] == agent_id
    
    def test_team_status_retrieval(self):
        """Test team status retrieval"""
        if not AUTOGEN_V04_AVAILABLE:
            pytest.skip("AutoGen v0.4 not available")
        
        adapter = AutoGenV04Adapter("team_status_test")
        
        # Test non-existent team
        status = adapter.get_team_status("non_existent")
        assert status["status"] == "not_found"
        
        # Create team and test status
        with patch('layers.adapter.autogen.adapter.create_llm_backend'):
            with patch('layers.adapter.autogen.adapter.AssistantAgent') as mock_agent:
                with patch('layers.adapter.autogen.adapter.RoundRobinGroupChat') as mock_team:
                    mock_agent_instance = Mock()
                    mock_agent_instance.name = "Test Agent"
                    mock_agent.return_value = mock_agent_instance
                    
                    mock_team_instance = Mock()
                    mock_team_instance.participants = [mock_agent_instance]
                    mock_team.return_value = mock_team_instance
                    
                    backend_id = adapter.create_llm_backend(
                        backend_id="client",
                        backend_type="openai",
                        api_key=self.test_api_key,
                        model="gpt-3.5-turbo"
                    )
                    agent_id = adapter.create_assistant_agent("agent", "agent", "Test", backend_id)
                    team_id = adapter._create_team_internal("team", [agent_id])
                    
                    status = adapter.get_team_status(team_id)
                    assert status["status"] == "active"
                    assert status["team_id"] == team_id
                    assert len(status["participants"]) == 1
    
    def test_adapter_status(self):
        """Test adapter status"""
        if not AUTOGEN_V04_AVAILABLE:
            pytest.skip("AutoGen v0.4 not available")
        
        adapter = AutoGenV04Adapter("adapter_status_test")
        status = adapter.get_adapter_status()
        
        assert status["adapter_name"] == "adapter_status_test"
        assert status["autogen_version"] == "0.4+"
        assert status["autogen_available"] == AUTOGEN_V04_AVAILABLE
        assert "agents_count" in status
        assert "teams_count" in status
        assert "model_clients_count" in status
    
    @pytest.mark.asyncio
    async def test_adapter_cleanup(self):
        """Test adapter cleanup and resource management"""
        if not AUTOGEN_V04_AVAILABLE:
            pytest.skip("AutoGen v0.4 not available")
        
        adapter = AutoGenV04Adapter("cleanup_test")
        
        # Mock model client with close method
        with patch('layers.adapter.autogen.adapter.create_llm_backend') as mock_client_class:
            mock_client = Mock()
            mock_client.close = AsyncMock()
            mock_client_class.return_value = mock_client
            
            # Create LLM backend
            adapter.create_llm_backend(
                backend_id="client",
                backend_type="openai",
                api_key=self.test_api_key,
                model="gpt-3.5-turbo"
            )
            
            # Test cleanup
            await adapter.close()
            
            # Verify close was called
            mock_client.close.assert_called_once()
    
    def test_convenience_function(self):
        """Test convenience function for adapter creation"""
        if not AUTOGEN_V04_AVAILABLE:
            pytest.skip("AutoGen v0.4 not available")
        
        with patch('layers.adapter.autogen.adapter.create_llm_backend'):
            adapter = create_autogen_v04_adapter(
                name="convenience_test",
                api_key=self.test_api_key,
                model="gpt-4",
                temperature=0.8
            )
            
            assert adapter.name == "convenience_test"
            assert adapter.default_model == "gpt-4"
            assert "default" in adapter.llm_backends


@pytest.mark.integration
@pytest.mark.api_key_required
class TestAutoGenV04AdapterIntegration:
    """Integration tests for AutoGen v0.4 adapter with real APIs"""
    
    @pytest.mark.asyncio
    async def test_real_deepseek_chat(self):
        """Test chat with real DeepSeek API"""
        if not AUTOGEN_V04_AVAILABLE:
            pytest.skip("AutoGen v0.4 not available")
        
        api_key = os.getenv("DEEPSEEK_API_KEY")
        if not api_key:
            pytest.skip("No DEEPSEEK_API_KEY for integration test")
        
        adapter = AutoGenV04Adapter("integration_test", {
            "backend_type": "deepseek",
            "default_model": "deepseek-chat"
        })
        
        try:
            # Create DeepSeek backend
            backend_id = adapter.create_llm_backend(
                backend_id="deepseek_test",
                backend_type="deepseek",
                api_key=api_key,
                model="deepseek-chat",
                temperature=0.1
            )
            
            # Create assistant agent
            agent_id = adapter.create_assistant_agent(
                "deepseek_agent",
                "deepseek_assistant",  # Must be a valid Python identifier
                "You are a helpful assistant. Be very brief.",
                backend_id
            )
            
            # Execute chat task
            task = UniversalTask(
                content="Say hello in exactly 3 words.",
                task_type=TaskType.CONVERSATION,
                priority=TaskPriority.NORMAL,
                requirements=TaskRequirements(),
                context={},
                task_id="deepseek_integration_test"
            )
            
            context = UniversalContext({'user_id': 'integration_user'})
            result = await asyncio.wait_for(
                adapter.execute_task(task, context),
                timeout=30.0
            )
            
            assert result.is_successful()
            assert "reply" in result.data
            assert len(result.data["reply"]) > 0
            assert result.data["agent_id"] == agent_id
            
        finally:
            await adapter.close()
    
    @pytest.mark.asyncio
    async def test_real_deepseek_team_chat(self):
        """Test team chat with real DeepSeek API"""
        if not AUTOGEN_V04_AVAILABLE:
            pytest.skip("AutoGen v0.4 not available")
        
        api_key = os.getenv("DEEPSEEK_API_KEY")
        if not api_key:
            pytest.skip("No DEEPSEEK_API_KEY for integration test")
        
        adapter = AutoGenV04Adapter("team_integration_test", {
            "backend_type": "deepseek",
            "default_model": "deepseek-chat"
        })
        
        try:
            # Create DeepSeek backend
            backend_id = adapter.create_llm_backend(
                backend_id="deepseek_team",
                backend_type="deepseek",
                api_key=api_key,
                model="deepseek-chat",
                temperature=0.1
            )
            
            # Create two assistant agents
            agent1_id = adapter.create_assistant_agent(
                "agent1",
                "creative_assistant",  # Must be a valid Python identifier
                "You are a creative assistant. Generate interesting ideas.",
                backend_id
            )
            
            agent2_id = adapter.create_assistant_agent(
                "agent2",
                "critical_assistant",  # Must be a valid Python identifier
                "You are a critical thinker. Analyze and improve ideas.",
                backend_id
            )
            
            # Create team
            team_id = adapter._create_team_internal(
                team_id="deepseek_team",
                agent_ids=[agent1_id, agent2_id],
                max_turns=3
            )
            
            # Execute collaboration task
            task = UniversalTask(
                content="Suggest a name for a new programming language and explain why it's good.",
                task_type=TaskType.TOOL_EXECUTION,
                priority=TaskPriority.NORMAL,
                requirements=TaskRequirements(),
                context={},
                task_id="deepseek_team_test"
            )
            
            context = UniversalContext({'user_id': 'integration_user'})
            result = await asyncio.wait_for(
                adapter.execute_task(task, context),
                timeout=60.0
            )
            
            assert result.status == ResultStatus.SUCCESS
            assert "team_id" in result.content
            assert result.content["team_id"] == team_id
            assert "conversation_log" in result.content
            assert len(result.content["conversation_log"]) > 0
            
        finally:
            await adapter.close()


if __name__ == "__main__":
    pytest.main([
        __file__,
        "-v",
        "--tb=short",
        "-x",
        "--durations=10"
    ]) 


#!/usr/bin/env python3
"""
ADC CLI Application
Agent Development Center 命令行界面主程序
"""

import argparse
import asyncio
import sys
import os
from pathlib import Path
from typing import Optional, List
import logging

# 添加项目根目录到路径
project_root = Path(__file__).parent.parent.parent.parent
sys.path.insert(0, str(project_root))

from .commands import CommandRegistry
from .interactive import InteractiveShell
from ...infrastructure.logging import get_logger
from ...infrastructure.config.config_manager import ConfigManager


class ADCCLIApp:
    """
    ADC CLI应用程序主类
    """
    
    def __init__(self):
        self.config_manager = ConfigManager()
        self.logger = get_logger("adc_cli")
        self.command_registry = CommandRegistry()
        self.interactive_shell = None
        
    def create_parser(self) -> argparse.ArgumentParser:
        """创建命令行参数解析器"""
        parser = argparse.ArgumentParser(
            prog="adc",
            description="Agent Development Center - 下一代AI Agent开发平台",
            formatter_class=argparse.RawDescriptionHelpFormatter,
            epilog="""
示例用法:
  adc --version                    # 显示版本信息
  adc --interactive                # 启动交互模式
  adc agent list                   # 列出所有Agent
  adc agent create --name MyAgent  # 创建新Agent
  adc workflow run --id workflow1  # 运行工作流
  adc team create --name MyTeam    # 创建团队
  adc demo openai                  # 运行OpenAI演示
  adc demo business                # 运行业务层演示

更多信息请访问: https://github.com/fpga1988/zeus
            """
        )
        
        # 全局选项
        parser.add_argument(
            "--version", "-v",
            action="version",
            version="Agent Development Center v2.0.0-alpha"
        )
        
        parser.add_argument(
            "--config", "-c",
            type=str,
            help="配置文件路径"
        )
        
        parser.add_argument(
            "--verbose", "-V",
            action="store_true",
            help="详细输出模式"
        )
        
        parser.add_argument(
            "--interactive", "-i",
            action="store_true",
            help="启动交互模式"
        )
        
        parser.add_argument(
            "--log-level",
            choices=["DEBUG", "INFO", "WARNING", "ERROR"],
            default="INFO",
            help="日志级别"
        )
        
        # 子命令
        subparsers = parser.add_subparsers(
            dest="command",
            help="可用命令",
            metavar="COMMAND"
        )
        
        # 注册所有命令
        self.command_registry.register_commands(subparsers)
        
        return parser
    
    def setup_logging(self, log_level: str, verbose: bool = False):
        """设置日志"""
        level = getattr(logging, log_level.upper())
        logging.basicConfig(
            level=level,
            format='%(asctime)s - %(name)s - %(levelname)s - %(message)s' if verbose else '%(message)s',
            handlers=[logging.StreamHandler(sys.stdout)]
        )
    
    async def run_async(self, args: argparse.Namespace) -> int:
        """异步运行应用程序"""
        try:
            # 设置日志
            self.setup_logging(args.log_level, args.verbose)
            
            # 加载配置
            if args.config:
                self.config_manager.load_config(args.config)
            
            # 交互模式
            if args.interactive:
                self.interactive_shell = InteractiveShell(self.command_registry)
                return await self.interactive_shell.run()
            
            # 命令模式
            if args.command:
                return await self.command_registry.execute_command(args)
            else:
                # 没有命令，显示帮助
                parser = self.create_parser()
                parser.print_help()
                return 0
                
        except KeyboardInterrupt:
            print("\n👋 用户中断，ADC已退出")
            return 130
        except Exception as e:
            self.logger.error(f"应用程序错误: {e}")
            if args.verbose:
                import traceback
                traceback.print_exc()
            return 1
    
    async def run_command(self, command_args: List[str]) -> bool:
        """运行指定命令 - 用于测试和程序化调用"""
        try:
            parser = self.create_parser()
            args = parser.parse_args(command_args)
            result = await self.run_async(args)
            return result == 0  # 成功返回True，失败返回False
        except SystemExit:
            # argparse在--help或--version时会抛出SystemExit
            return True
        except Exception as e:
            self.logger.error(f"命令执行错误: {e}")
            return False
    
    def run(self, argv: Optional[List[str]] = None) -> int:
        """运行应用程序"""
        parser = self.create_parser()
        args = parser.parse_args(argv)
        
        # 运行异步主函数
        try:
            return asyncio.run(self.run_async(args))
        except KeyboardInterrupt:
            return 130


def main():
    """CLI入口点"""
    app = ADCCLIApp()
    sys.exit(app.run())


if __name__ == "__main__":
    main() 


"""
CollaborationManager单元测试 - 修复版
测试协作管理器的实际功能接口
"""

import pytest
from unittest.mock import Mock, AsyncMock
from datetime import datetime

from layers.business.teams.collaboration_manager import (
    CollaborationManager, TeamMember, CollaborationRole, 
    CollaborationResult, CollaborationPattern
)
from layers.framework.abstractions.agent import UniversalAgent, AgentCapability
from layers.framework.abstractions.task import UniversalTask, TaskType


class TestCollaborationManagerFixed:
    """测试CollaborationManager类的实际实现"""
    
    def setup_method(self):
        """每个测试方法前的设置"""
        self.manager = CollaborationManager()
        
        # 创建模拟的Agent
        self.mock_agent1 = Mock(spec=UniversalAgent)
        self.mock_agent1.agent_id = "agent_001"
        self.mock_agent1.name = "Test Agent 1"
        self.mock_agent1.get_capabilities.return_value = [AgentCapability.REASONING]
        
        self.mock_agent2 = Mock(spec=UniversalAgent)
        self.mock_agent2.agent_id = "agent_002"
        self.mock_agent2.name = "Test Agent 2"
        self.mock_agent2.get_capabilities.return_value = [AgentCapability.PLANNING]
        
        self.mock_agent3 = Mock(spec=UniversalAgent)
        self.mock_agent3.agent_id = "agent_003"
        self.mock_agent3.name = "Test Agent 3"
        self.mock_agent3.get_capabilities.return_value = [AgentCapability.CODE_GENERATION]
        
        # 创建团队成员
        self.mock_member1 = TeamMember(
            agent=self.mock_agent1,
            role=CollaborationRole.EXPERT,
            capabilities=[AgentCapability.REASONING]
        )
        
        self.mock_member2 = TeamMember(
            agent=self.mock_agent2,
            role=CollaborationRole.CONTRIBUTOR,
            capabilities=[AgentCapability.PLANNING]
        )
        
        self.mock_member3 = TeamMember(
            agent=self.mock_agent3,
            role=CollaborationRole.LEADER,
            capabilities=[AgentCapability.CODE_GENERATION]
        )
    
    def test_manager_initialization(self):
        """测试管理器初始化"""
        assert self.manager.teams == {}
        assert self.manager.active_collaborations == {}
        assert self.manager.collaboration_history == []
        assert len(self.manager.pattern_handlers) == 8  # 8种协作模式
    
    @pytest.mark.asyncio
    async def test_create_team_with_members(self):
        """测试创建包含成员的团队"""
        team_id = "test_team_001"
        members = [self.mock_member1, self.mock_member2]
        
        success = await self.manager.create_team(team_id, members)
        assert success is True
        
        # 验证团队创建
        assert team_id in self.manager.teams
        team_members = self.manager.teams[team_id]
        assert len(team_members) == 2
        
        # 验证成员关系 - 使用agent_id作为key
        assert "agent_001" in team_members
        assert "agent_002" in team_members
        assert team_members["agent_001"] == self.mock_member1
        assert team_members["agent_002"] == self.mock_member2
    
    @pytest.mark.asyncio
    async def test_create_team_without_members(self):
        """测试创建空团队"""
        team_id = "empty_team"
        
        success = await self.manager.create_team(team_id, [])
        assert success is True
        
        assert team_id in self.manager.teams
        assert len(self.manager.teams[team_id]) == 0
    
    @pytest.mark.asyncio
    async def test_create_team_duplicate_id(self):
        """测试创建重复ID的团队"""
        team_id = "duplicate_team"
        
        # 第一次创建
        success1 = await self.manager.create_team(team_id, [])
        assert success1 is True
        
        # 第二次创建相同ID应该失败
        success2 = await self.manager.create_team(team_id, [])
        assert success2 is False
    
    @pytest.mark.asyncio
    async def test_add_team_member(self):
        """测试添加团队成员"""
        team_id = "test_team"
        member_id = "agent_001"
        
        success = await self.manager.add_team_member(
            team_id, member_id, self.mock_agent1, CollaborationRole.EXPERT
        )
        
        assert success is True
        assert team_id in self.manager.teams
        assert member_id in self.manager.teams[team_id]
        
        member = self.manager.teams[team_id][member_id]
        assert member.agent == self.mock_agent1
        assert member.role == CollaborationRole.EXPERT
    
    def test_remove_team_member_success(self):
        """测试成功移除团队成员"""
        team_id = "test_team"
        member_id = "agent_001"
        
        # 先添加团队和成员
        self.manager.teams[team_id] = {member_id: self.mock_member1}
        
        # 移除成员
        success = self.manager.remove_team_member(team_id, member_id)
        assert success is True
        assert member_id not in self.manager.teams[team_id]
    
    def test_remove_team_member_nonexistent(self):
        """测试移除不存在的团队成员"""
        success = self.manager.remove_team_member("nonexistent_team", "nonexistent_member")
        assert success is False
    
    def test_get_team_members(self):
        """测试获取团队成员"""
        team_id = "test_team"
        
        # 添加团队和成员
        self.manager.teams[team_id] = {
            "agent_001": self.mock_member1,
            "agent_002": self.mock_member2
        }
        
        members = self.manager.get_team_members(team_id)
        assert len(members) == 2
        assert "agent_001" in members
        assert "agent_002" in members
    
    def test_get_team_members_nonexistent(self):
        """测试获取不存在团队的成员"""
        members = self.manager.get_team_members("nonexistent_team")
        assert members == {}
    
    @pytest.mark.asyncio
    async def test_execute_collaboration_basic(self):
        """测试基础协作执行"""
        team_id = "test_team"
        
        # 创建团队和成员
        await self.manager.create_team(team_id, [self.mock_member1, self.mock_member2])
        
        # 创建测试任务
        task = UniversalTask(content="Test collaborative task", task_type=TaskType.ANALYSIS)
        
        # Mock agent的execute方法
        self.mock_agent1.execute = AsyncMock(return_value=Mock(status="success"))
        self.mock_agent2.execute = AsyncMock(return_value=Mock(status="success"))
        
        # 执行协作
        result = await self.manager.execute_collaboration(
            team_id, task, CollaborationPattern.PARALLEL
        )
        
        assert isinstance(result, CollaborationResult)
        assert result.task_id is not None
        assert result.pattern == CollaborationPattern.PARALLEL
    
    @pytest.mark.asyncio
    async def test_execute_collaboration_nonexistent_team(self):
        """测试在不存在的团队上执行协作"""
        task = UniversalTask(content="Test task", task_type=TaskType.PLANNING)
        
        with pytest.raises(ValueError, match="Team nonexistent_team not found"):
            await self.manager.execute_collaboration(
                "nonexistent_team", task, CollaborationPattern.SEQUENTIAL
            )
    
    @pytest.mark.asyncio
    async def test_collaborate_alias_method(self):
        """测试collaborate方法（execute_collaboration的别名）"""
        team_id = "test_team"
        
        # 创建团队和成员
        await self.manager.create_team(team_id, [self.mock_member1])
        
        # 创建测试任务
        task = UniversalTask(content="Test task", task_type=TaskType.CUSTOM)
        
        # Mock agent的execute方法
        self.mock_agent1.execute = AsyncMock(return_value=Mock(status="success"))
        
        # 测试collaborate方法
        result = await self.manager.collaborate(team_id, task, CollaborationPattern.SEQUENTIAL)
        
        assert isinstance(result, CollaborationResult)
        assert result.pattern == CollaborationPattern.SEQUENTIAL
    
    def test_collaboration_patterns_registered(self):
        """测试所有协作模式都已注册"""
        expected_patterns = [
            CollaborationPattern.SEQUENTIAL,
            CollaborationPattern.PARALLEL,
            CollaborationPattern.ROUND_ROBIN,
            CollaborationPattern.EXPERT_CONSULTATION,
            CollaborationPattern.PEER_REVIEW,
            CollaborationPattern.DEBATE,
            CollaborationPattern.CONSENSUS,
            CollaborationPattern.HIERARCHICAL,
        ]
        
        for pattern in expected_patterns:
            assert pattern in self.manager.pattern_handlers
            assert callable(self.manager.pattern_handlers[pattern]) 


"""
CLI Application Module
命令行界面应用模块
"""

from .main import ADCCLIApp
from .commands import CommandRegistry
from .interactive import InteractiveShell

__all__ = [
    'ADCCLIApp',
    'CommandRegistry', 
    'InteractiveShell',
] 


"""
ADC平台核心装饰器系统
提供能力声明、知识增强、上下文感知等功能装饰器

设计理念：
- 声明式编程：通过装饰器声明Agent能力和特性
- 知识库优先：自动集成知识库检索和增强
- 上下文感知：智能管理对话和任务上下文
"""

import asyncio
import functools
import logging
import time
from typing import Dict, List, Any, Optional, Callable, Union
from dataclasses import dataclass, field
from datetime import datetime
from enum import Enum

logger = logging.getLogger(__name__)


class CapabilityType(Enum):
    """能力类型枚举"""
    DESIGN = "design"           # 设计能力
    VERIFICATION = "verification"  # 验证能力
    OPTIMIZATION = "optimization"  # 优化能力
    ANALYSIS = "analysis"       # 分析能力
    GENERATION = "generation"   # 生成能力
    DEBUGGING = "debugging"     # 调试能力
    CONSULTATION = "consultation"  # 咨询能力


class KnowledgeDomain(Enum):
    """知识领域枚举"""
    FPGA = "fpga"              # FPGA相关知识
    HDL = "hdl"                # HDL语言知识
    VERIFICATION = "verification"  # 验证方法学
    PROTOCOLS = "protocols"     # 通信协议
    OPTIMIZATION = "optimization"  # 优化技术
    DEBUGGING = "debugging"     # 调试技术
    TOOLS = "tools"            # EDA工具


@dataclass
class CapabilityMetadata:
    """能力元数据"""
    name: str
    capability_type: CapabilityType
    description: str
    input_types: List[str] = field(default_factory=list)
    output_types: List[str] = field(default_factory=list)
    prerequisites: List[str] = field(default_factory=list)
    confidence_threshold: float = 0.7
    max_retry_count: int = 3
    timeout_seconds: int = 30
    created_at: datetime = field(default_factory=datetime.now)


@dataclass
class KnowledgeEnhancementConfig:
    """知识增强配置"""
    domains: List[KnowledgeDomain]
    retrieval_count: int = 5
    confidence_threshold: float = 0.6
    enable_caching: bool = True
    cache_ttl_seconds: int = 3600
    enable_context_injection: bool = True
    context_window_size: int = 2000


@dataclass
class ContextAwarenessConfig:
    """上下文感知配置"""
    enable_conversation_history: bool = True
    history_window_size: int = 10
    enable_task_context: bool = True
    enable_user_preferences: bool = True
    enable_session_memory: bool = True
    context_expiry_seconds: int = 7200


# 全局注册表
_capability_registry: Dict[str, CapabilityMetadata] = {}
_method_capabilities: Dict[str, List[str]] = {}


def capability(
    name: str,
    capability_type: CapabilityType,
    description: str,
    input_types: List[str] = None,
    output_types: List[str] = None,
    prerequisites: List[str] = None,
    confidence_threshold: float = 0.7,
    max_retry_count: int = 3,
    timeout_seconds: int = 30
):
    """
    能力装饰器 - 声明方法的能力特性
    
    Args:
        name: 能力名称
        capability_type: 能力类型
        description: 能力描述
        input_types: 输入类型列表
        output_types: 输出类型列表
        prerequisites: 前置条件
        confidence_threshold: 置信度阈值
        max_retry_count: 最大重试次数
        timeout_seconds: 超时时间
    
    Example:
        @capability(
            name="verilog_code_generation",
            capability_type=CapabilityType.GENERATION,
            description="生成Verilog HDL代码",
            input_types=["natural_language", "specification"],
            output_types=["verilog_code", "testbench"],
            confidence_threshold=0.8
        )
        async def generate_verilog_code(self, request: str) -> str:
            # 实现代码生成逻辑
            pass
    """
    def decorator(func: Callable) -> Callable:
        # 创建能力元数据
        metadata = CapabilityMetadata(
            name=name,
            capability_type=capability_type,
            description=description,
            input_types=input_types or [],
            output_types=output_types or [],
            prerequisites=prerequisites or [],
            confidence_threshold=confidence_threshold,
            max_retry_count=max_retry_count,
            timeout_seconds=timeout_seconds
        )
        
        # 注册能力
        _capability_registry[name] = metadata
        
        # 记录方法的能力
        method_key = f"{func.__module__}.{func.__qualname__}"
        if method_key not in _method_capabilities:
            _method_capabilities[method_key] = []
        _method_capabilities[method_key].append(name)
        
        @functools.wraps(func)
        async def wrapper(*args, **kwargs):
            start_time = time.time()
            retry_count = 0
            
            logger.info(f"🚀 执行能力: {name} ({capability_type.value})")
            
            while retry_count < max_retry_count:
                try:
                    # 设置超时
                    result = await asyncio.wait_for(
                        func(*args, **kwargs),
                        timeout=timeout_seconds
                    )
                    
                    execution_time = time.time() - start_time
                    logger.info(f"✅ 能力执行成功: {name} (耗时: {execution_time:.2f}s)")
                    
                    # 添加执行元数据
                    if hasattr(result, '__dict__'):
                        result._capability_metadata = {
                            'capability_name': name,
                            'execution_time': execution_time,
                            'retry_count': retry_count,
                            'confidence': getattr(result, 'confidence', 1.0)
                        }
                    
                    return result
                    
                except asyncio.TimeoutError:
                    retry_count += 1
                    logger.warning(f"⏰ 能力执行超时: {name} (重试 {retry_count}/{max_retry_count})")
                    if retry_count >= max_retry_count:
                        raise
                        
                except Exception as e:
                    retry_count += 1
                    logger.error(f"❌ 能力执行失败: {name} (重试 {retry_count}/{max_retry_count}): {e}")
                    if retry_count >= max_retry_count:
                        raise
                    
                    # 短暂等待后重试
                    await asyncio.sleep(0.5 * retry_count)
            
        # 保存元数据到函数属性
        wrapper._capability_metadata = metadata
        wrapper._is_capability = True
        
        return wrapper
    
    return decorator


def knowledge_enhanced(
    domains: List[KnowledgeDomain],
    retrieval_count: int = 5,
    confidence_threshold: float = 0.6,
    enable_caching: bool = True,
    cache_ttl_seconds: int = 3600,
    enable_context_injection: bool = True,
    context_window_size: int = 2000
):
    """
    知识增强装饰器 - 自动检索和注入相关知识
    
    Args:
        domains: 知识领域列表
        retrieval_count: 检索数量
        confidence_threshold: 置信度阈值
        enable_caching: 启用缓存
        cache_ttl_seconds: 缓存TTL
        enable_context_injection: 启用上下文注入
        context_window_size: 上下文窗口大小
    
    Example:
        @knowledge_enhanced(
            domains=[KnowledgeDomain.FPGA, KnowledgeDomain.HDL],
            retrieval_count=3,
            confidence_threshold=0.7
        )
        async def design_fpga_module(self, specification: str) -> str:
            # 方法执行前会自动检索相关FPGA和HDL知识
            # 知识会注入到self.enhanced_context中
            pass
    """
    def decorator(func: Callable) -> Callable:
        config = KnowledgeEnhancementConfig(
            domains=domains,
            retrieval_count=retrieval_count,
            confidence_threshold=confidence_threshold,
            enable_caching=enable_caching,
            cache_ttl_seconds=cache_ttl_seconds,
            enable_context_injection=enable_context_injection,
            context_window_size=context_window_size
        )
        
        @functools.wraps(func)
        async def wrapper(self, *args, **kwargs):
            logger.info(f"🧠 启用知识增强: {func.__name__} (领域: {[d.value for d in domains]})")
            
            try:
                # 提取查询内容
                query_text = _extract_query_from_args(args, kwargs)
                
                # 检索相关知识
                if hasattr(self, 'knowledge_service'):
                    enhanced_knowledge = await self._retrieve_knowledge(
                        query_text, config
                    )
                    
                    # 注入增强上下文
                    if enable_context_injection:
                        self.enhanced_context = enhanced_knowledge
                        logger.debug(f"💡 已注入 {len(enhanced_knowledge)} 条知识")
                
                # 执行原始方法
                result = await func(self, *args, **kwargs)
                
                # 清理增强上下文
                if hasattr(self, 'enhanced_context'):
                    delattr(self, 'enhanced_context')
                
                return result
                
            except Exception as e:
                logger.error(f"❌ 知识增强失败: {e}")
                # 降级执行原始方法
                return await func(self, *args, **kwargs)
        
        # 保存配置到函数属性
        wrapper._knowledge_enhancement_config = config
        wrapper._is_knowledge_enhanced = True
        
        return wrapper
    
    return decorator


def context_aware(
    enable_conversation_history: bool = True,
    history_window_size: int = 10,
    enable_task_context: bool = True,
    enable_user_preferences: bool = True,
    enable_session_memory: bool = True,
    context_expiry_seconds: int = 7200
):
    """
    上下文感知装饰器 - 自动管理和注入上下文信息
    
    Args:
        enable_conversation_history: 启用对话历史
        history_window_size: 历史窗口大小
        enable_task_context: 启用任务上下文
        enable_user_preferences: 启用用户偏好
        enable_session_memory: 启用会话记忆
        context_expiry_seconds: 上下文过期时间
    
    Example:
        @context_aware(
            enable_conversation_history=True,
            history_window_size=5,
            enable_task_context=True
        )
        async def continue_design_task(self, new_requirement: str) -> str:
            # 方法执行前会自动加载相关上下文
            # 上下文信息可通过self.context访问
            pass
    """
    def decorator(func: Callable) -> Callable:
        config = ContextAwarenessConfig(
            enable_conversation_history=enable_conversation_history,
            history_window_size=history_window_size,
            enable_task_context=enable_task_context,
            enable_user_preferences=enable_user_preferences,
            enable_session_memory=enable_session_memory,
            context_expiry_seconds=context_expiry_seconds
        )
        
        @functools.wraps(func)
        async def wrapper(self, *args, **kwargs):
            logger.info(f"🎯 启用上下文感知: {func.__name__}")
            
            try:
                # 加载上下文
                if hasattr(self, 'context_manager'):
                    context = await self._load_context(config)
                    self.current_context = context
                    logger.debug(f"📝 已加载上下文: {len(context.get('history', []))} 条历史")
                
                # 执行原始方法
                result = await func(self, *args, **kwargs)
                
                # 更新上下文
                if hasattr(self, 'context_manager'):
                    await self._update_context(func.__name__, args, kwargs, result)
                
                return result
                
            except Exception as e:
                logger.error(f"❌ 上下文感知失败: {e}")
                # 降级执行原始方法
                return await func(self, *args, **kwargs)
        
        # 保存配置到函数属性
        wrapper._context_awareness_config = config
        wrapper._is_context_aware = True
        
        return wrapper
    
    return decorator


# 辅助函数

def _extract_query_from_args(args, kwargs) -> str:
    """从参数中提取查询文本"""
    query_parts = []
    
    # 从位置参数提取
    for arg in args:
        if isinstance(arg, str) and len(arg) > 10:  # 假设有意义的查询至少10个字符
            query_parts.append(arg)
    
    # 从关键字参数提取
    for key, value in kwargs.items():
        if isinstance(value, str) and len(value) > 10:
            query_parts.append(f"{key}: {value}")
    
    return " ".join(query_parts)


def get_capability_metadata(capability_name: str) -> Optional[CapabilityMetadata]:
    """获取能力元数据"""
    return _capability_registry.get(capability_name)


def get_method_capabilities(method_key: str) -> List[str]:
    """获取方法的能力列表"""
    return _method_capabilities.get(method_key, [])


def list_all_capabilities() -> Dict[str, CapabilityMetadata]:
    """列出所有注册的能力"""
    return _capability_registry.copy()


def capability_exists(capability_name: str) -> bool:
    """检查能力是否存在"""
    return capability_name in _capability_registry


# 能力发现装饰器
def discoverable(category: str = "general", priority: int = 0):
    """
    可发现装饰器 - 标记方法为可被动态发现的能力
    
    Args:
        category: 能力分类
        priority: 优先级（数字越大优先级越高）
    """
    def decorator(func: Callable) -> Callable:
        func._is_discoverable = True
        func._discovery_category = category
        func._discovery_priority = priority
        return func
    
    return decorator


# 性能监控装饰器
def performance_monitored(
    enable_timing: bool = True,
    enable_memory_tracking: bool = False,
    enable_metrics_collection: bool = True
):
    """
    性能监控装饰器 - 自动收集方法执行的性能指标
    
    Args:
        enable_timing: 启用执行时间监控
        enable_memory_tracking: 启用内存使用监控
        enable_metrics_collection: 启用指标收集
    """
    def decorator(func: Callable) -> Callable:
        @functools.wraps(func)
        async def wrapper(*args, **kwargs):
            start_time = time.time()
            
            if enable_timing:
                logger.debug(f"⏱️ 开始执行: {func.__name__}")
            
            try:
                result = await func(*args, **kwargs)
                
                if enable_timing:
                    execution_time = time.time() - start_time
                    logger.info(f"⏱️ 执行完成: {func.__name__} (耗时: {execution_time:.3f}s)")
                
                return result
                
            except Exception as e:
                if enable_timing:
                    execution_time = time.time() - start_time
                    logger.error(f"⏱️ 执行失败: {func.__name__} (耗时: {execution_time:.3f}s): {e}")
                raise
        
        wrapper._is_performance_monitored = True
        return wrapper
    
    return decorator


# 组合装饰器 - 常用组合
def fpga_expert_method(
    capability_name: str,
    capability_type: CapabilityType,
    description: str,
    knowledge_domains: List[KnowledgeDomain] = None,
    enable_context: bool = True
):
    """
    FPGA专家方法装饰器 - 组合常用装饰器
    
    这是一个便利装饰器，组合了capability、knowledge_enhanced和context_aware
    """
    def decorator(func: Callable) -> Callable:
        # 应用多个装饰器
        enhanced_func = func
        
        # 1. 性能监控
        enhanced_func = performance_monitored()(enhanced_func)
        
        # 2. 上下文感知
        if enable_context:
            enhanced_func = context_aware()(enhanced_func)
        
        # 3. 知识增强
        if knowledge_domains:
            enhanced_func = knowledge_enhanced(domains=knowledge_domains)(enhanced_func)
        
        # 4. 能力声明
        enhanced_func = capability(
            name=capability_name,
            capability_type=capability_type,
            description=description
        )(enhanced_func)
        
        # 5. 可发现性
        enhanced_func = discoverable(category="fpga_expert")(enhanced_func)
        
        return enhanced_func
    
    return decorator 


"""
知识管理组件

负责知识的存储、检索、更新和维护
"""

from typing import Dict, Any, List, Optional, Set
from dataclasses import dataclass, field
from datetime import datetime
from enum import Enum

from ..framework.abstractions.context import UniversalContext
from ..framework.abstractions.task import UniversalTask, TaskType


class KnowledgeType(Enum):
    """知识类型"""
    FACT = "fact"  # 事实
    RULE = "rule"  # 规则
    CONCEPT = "concept"  # 概念
    PROCEDURE = "procedure"  # 过程
    PATTERN = "pattern"  # 模式


class KnowledgeSource(Enum):
    """知识来源"""
    SYSTEM = "system"  # 系统内置
    USER = "user"  # 用户提供
    LEARNED = "learned"  # 学习获得
    INFERRED = "inferred"  # 推理得出


class KnowledgeStatus(Enum):
    """知识状态"""
    ACTIVE = "active"  # 活跃
    INACTIVE = "inactive"  # 非活跃
    DEPRECATED = "deprecated"  # 已弃用
    PENDING = "pending"  # 待验证


@dataclass
class KnowledgeItem:
    """知识项"""
    id: str
    type: KnowledgeType
    content: Dict[str, Any]
    source: KnowledgeSource
    status: KnowledgeStatus = KnowledgeStatus.ACTIVE
    tags: Set[str] = field(default_factory=set)
    metadata: Dict[str, Any] = field(default_factory=dict)
    created_at: datetime = field(default_factory=datetime.now)
    updated_at: datetime = field(default_factory=datetime.now)
    version: int = 1
    
    def update(self, content: Dict[str, Any], source: Optional[KnowledgeSource] = None):
        """更新知识项"""
        self.content = content
        if source:
            self.source = source
        self.updated_at = datetime.now()
        self.version += 1
    
    def add_tags(self, *tags: str):
        """添加标签"""
        self.tags.update(tags)
        self.updated_at = datetime.now()
    
    def remove_tags(self, *tags: str):
        """移除标签"""
        self.tags.difference_update(tags)
        self.updated_at = datetime.now()
    
    def set_status(self, status: KnowledgeStatus):
        """设置状态"""
        self.status = status
        self.updated_at = datetime.now()
    
    def set_metadata(self, key: str, value: Any):
        """设置元数据"""
        self.metadata[key] = value
        self.updated_at = datetime.now()


class KnowledgeManagement:
    """
    知识管理组件
    
    负责知识的存储、检索、更新和维护
    """
    
    def __init__(self, config: Optional[Dict[str, Any]] = None):
        """初始化知识管理组件"""
        self.config = config or {}
        
        # 知识库
        self.knowledge_base: Dict[str, KnowledgeItem] = {}
        
        # 索引
        self.type_index: Dict[KnowledgeType, Set[str]] = {
            type_: set() for type_ in KnowledgeType
        }
        self.source_index: Dict[KnowledgeSource, Set[str]] = {
            source: set() for source in KnowledgeSource
        }
        self.status_index: Dict[KnowledgeStatus, Set[str]] = {
            status: set() for status in KnowledgeStatus
        }
        self.tag_index: Dict[str, Set[str]] = {}
        
        # 性能指标
        self.metrics = {
            'total_items': 0,
            'total_updates': 0,
            'total_queries': 0,
            'average_query_time': 0.0,
            'cache_hit_rate': 1.0,
            'type_distribution': {},
            'source_distribution': {},
            'status_distribution': {}
        }
        
        # 加载初始知识
        self._load_initial_knowledge()
    
    def _load_initial_knowledge(self):
        """加载初始知识"""
        # 加载系统知识
        system_knowledge = {
            'task_types': KnowledgeItem(
                id='system.task_types',
                type=KnowledgeType.CONCEPT,
                content={
                    'types': [type_.value for type_ in TaskType],
                    'descriptions': {
                        TaskType.CONVERSATION.value: '对话任务',
                        TaskType.CODE_GENERATION.value: '代码生成任务',
                        TaskType.CODE_EXECUTION.value: '代码执行任务',
                        TaskType.WEB_SEARCH.value: '网络搜索任务',
                        TaskType.FILE_OPERATION.value: '文件操作任务',
                        TaskType.CUSTOM.value: '自定义任务'
                    }
                },
                source=KnowledgeSource.SYSTEM,
                tags={'system', 'task', 'type'}
            ),
            'knowledge_types': KnowledgeItem(
                id='system.knowledge_types',
                type=KnowledgeType.CONCEPT,
                content={
                    'types': [type_.value for type_ in KnowledgeType],
                    'descriptions': {
                        KnowledgeType.FACT.value: '事实知识',
                        KnowledgeType.RULE.value: '规则知识',
                        KnowledgeType.CONCEPT.value: '概念知识',
                        KnowledgeType.PROCEDURE.value: '过程知识',
                        KnowledgeType.PATTERN.value: '模式知识'
                    }
                },
                source=KnowledgeSource.SYSTEM,
                tags={'system', 'knowledge', 'type'}
            )
        }
        
        # 添加系统知识
        for item in system_knowledge.values():
            self.add_knowledge(item)
    
    async def manage_knowledge(
        self, 
                             context: UniversalContext, 
        task: UniversalTask
    ) -> UniversalContext:
        """
        管理知识
        
        Args:
            context: 输入上下文
            task: 相关任务
            
        Returns:
            处理后的上下文
        """
        import time
        start_time = time.time()
        
        try:
            # 1. 提取知识
            knowledge = self._extract_knowledge(context, task)
            
            # 2. 存储知识
            for item in knowledge:
                self.add_knowledge(item)
            
            # 3. 检索相关知识
            relevant_knowledge = self._retrieve_knowledge(context, task)
            
            # 4. 更新上下文
            managed_context = self._update_context(context, relevant_knowledge)
            
            # 更新指标
            processing_time = time.time() - start_time
            self._update_metrics(len(knowledge), len(relevant_knowledge), processing_time)
            
            return managed_context
            
        except Exception as e:
            # 错误处理
            return context
    
    def add_knowledge(self, item: KnowledgeItem):
        """添加知识项"""
        # 更新知识库
        self.knowledge_base[item.id] = item
        
        # 更新索引
        self.type_index[item.type].add(item.id)
        self.source_index[item.source].add(item.id)
        self.status_index[item.status].add(item.id)
        
        for tag in item.tags:
            if tag not in self.tag_index:
                self.tag_index[tag] = set()
            self.tag_index[tag].add(item.id)
        
        # 更新指标
        self.metrics['total_items'] += 1
        self._update_distribution_metrics()
    
    def update_knowledge(
        self, 
        id: str, 
        content: Dict[str, Any], 
        source: Optional[KnowledgeSource] = None
    ):
        """更新知识项"""
        if id not in self.knowledge_base:
            return
        
        item = self.knowledge_base[id]
        
        # 更新索引
        if source and source != item.source:
            self.source_index[item.source].remove(id)
            self.source_index[source].add(id)
        
        # 更新知识项
        item.update(content, source)
        
        # 更新指标
        self.metrics['total_updates'] += 1
    
    def remove_knowledge(self, id: str):
        """移除知识项"""
        if id not in self.knowledge_base:
            return
        
        item = self.knowledge_base[id]
        
        # 更新索引
        self.type_index[item.type].remove(id)
        self.source_index[item.source].remove(id)
        self.status_index[item.status].remove(id)
        
        for tag in item.tags:
            if tag in self.tag_index:
                self.tag_index[tag].remove(id)
                if not self.tag_index[tag]:
                    del self.tag_index[tag]
        
        # 移除知识项
        del self.knowledge_base[id]
        
        # 更新指标
        self.metrics['total_items'] -= 1
        self._update_distribution_metrics()
    
    def get_knowledge(self, id: str) -> Optional[KnowledgeItem]:
        """获取知识项"""
        return self.knowledge_base.get(id)
    
    def search_knowledge(
        self,
        type: Optional[KnowledgeType] = None,
        source: Optional[KnowledgeSource] = None,
        status: Optional[KnowledgeStatus] = None,
        tags: Optional[Set[str]] = None
    ) -> List[KnowledgeItem]:
        """搜索知识"""
        # 获取候选集
        candidates = set(self.knowledge_base.keys())
        
        # 按类型过滤
        if type:
            candidates &= self.type_index[type]
        
        # 按来源过滤
        if source:
            candidates &= self.source_index[source]
        
        # 按状态过滤
        if status:
            candidates &= self.status_index[status]
        
        # 按标签过滤
        if tags:
            for tag in tags:
                if tag in self.tag_index:
                    candidates &= self.tag_index[tag]
                else:
                    return []
        
        # 返回结果
        return [self.knowledge_base[id] for id in candidates]
    
    def _extract_knowledge(
        self, 
        context: UniversalContext, 
        task: UniversalTask
    ) -> List[KnowledgeItem]:
        """提取知识"""
        knowledge = []
        
        # 从任务中提取知识
        task_content = task.content
        if isinstance(task_content, dict):
            message = task_content.get('message', '')
            metadata = task_content.get('metadata', {})
            
            # 提取事实知识
            if message:
                knowledge.append(KnowledgeItem(
                    id=f'task.fact.{hash(message)}',
                    type=KnowledgeType.FACT,
                    content={'message': message},
                    source=KnowledgeSource.USER,
                    tags={'task', 'fact'}
                ))
            
            # 提取元数据知识
            if metadata:
                knowledge.append(KnowledgeItem(
                    id=f'task.metadata.{hash(str(metadata))}',
                    type=KnowledgeType.FACT,
                    content={'metadata': metadata},
                    source=KnowledgeSource.USER,
                    tags={'task', 'metadata'}
                ))
        
        # 从上下文中提取知识
        context_data = context.data
        for key, value in context_data.items():
            # 提取概念知识
            if key in ['template_name', 'enhancement_mode']:
                knowledge.append(KnowledgeItem(
                    id=f'context.concept.{key}.{hash(str(value))}',
                    type=KnowledgeType.CONCEPT,
                    content={key: value},
                    source=KnowledgeSource.SYSTEM,
                    tags={'context', 'concept', key}
                ))
            
            # 提取规则知识
            elif key in ['rules', 'constraints']:
                knowledge.append(KnowledgeItem(
                    id=f'context.rule.{key}.{hash(str(value))}',
                    type=KnowledgeType.RULE,
                    content={key: value},
                    source=KnowledgeSource.SYSTEM,
                    tags={'context', 'rule', key}
                ))
            
            # 提取过程知识
            elif key in ['steps', 'workflow']:
                knowledge.append(KnowledgeItem(
                    id=f'context.procedure.{key}.{hash(str(value))}',
                    type=KnowledgeType.PROCEDURE,
                    content={key: value},
                    source=KnowledgeSource.SYSTEM,
                    tags={'context', 'procedure', key}
                ))
            
            # 提取模式知识
            elif key in ['patterns', 'templates']:
                knowledge.append(KnowledgeItem(
                    id=f'context.pattern.{key}.{hash(str(value))}',
                    type=KnowledgeType.PATTERN,
                    content={key: value},
                    source=KnowledgeSource.SYSTEM,
                    tags={'context', 'pattern', key}
                ))
        
        return knowledge
    
    def _retrieve_knowledge(
        self, 
        context: UniversalContext, 
        task: UniversalTask
    ) -> List[KnowledgeItem]:
        """检索知识"""
        # 获取任务相关标签
        tags = {'task'}
        if isinstance(task.content, dict):
            metadata = task.content.get('metadata', {})
            tags.update(metadata.keys())
        
        # 获取上下文相关标签
        context_data = context.data
        for key in context_data.keys():
            tags.add(key)
        
        # 搜索相关知识
        all_knowledge = self.search_knowledge(
            status=KnowledgeStatus.ACTIVE,
            tags=None  # 暂时不使用标签过滤
        )
        
        # 排除系统知识
        return [
            item for item in all_knowledge
            if not (
                item.source == KnowledgeSource.SYSTEM and
                item.id.startswith('system.')
            )
        ]
    
    def _update_context(
        self, 
        context: UniversalContext, 
        knowledge: List[KnowledgeItem]
    ) -> UniversalContext:
        """更新上下文"""
        # 创建新的上下文
        managed_context = UniversalContext({})
        
        # 复制原始数据
        for key in context.data.keys():
            managed_context.set(key, context.get(key))
        
        # 添加知识
        knowledge_data = {}
        for item in knowledge:
            knowledge_data[item.id] = {
                'type': item.type.value,
                'content': item.content,
                'source': item.source.value,
                'tags': list(item.tags),
                'version': item.version,
                'updated_at': item.updated_at.isoformat()
            }
        
        managed_context.set('knowledge', knowledge_data)
        managed_context.set('knowledge_updated_at', datetime.now().isoformat())
        
        return managed_context
    
    def _update_metrics(self, extracted: int, retrieved: int, processing_time: float):
        """更新指标"""
        # 更新查询次数和时间
        self.metrics['total_queries'] += 1
        current_avg = self.metrics['average_query_time']
        total = self.metrics['total_queries']
        self.metrics['average_query_time'] = (
            (current_avg * (total - 1) + processing_time) / total
        )
    
    def _update_distribution_metrics(self):
        """更新分布指标"""
        total = max(self.metrics['total_items'], 1)
        
        # 更新类型分布
        self.metrics['type_distribution'] = {
            type_.value.lower(): len(items) / total
            for type_, items in self.type_index.items()
        }
        
        # 更新来源分布
        self.metrics['source_distribution'] = {
            source.value.lower(): len(items) / total
            for source, items in self.source_index.items()
        }
        
        # 更新状态分布
        self.metrics['status_distribution'] = {
            status.value.lower(): len(items) / total
            for status, items in self.status_index.items()
        }
    
    def get_metrics(self) -> Dict[str, Any]:
        """获取性能指标"""
        return self.metrics.copy()
    
    def get_status(self) -> Dict[str, Any]:
        """获取组件状态"""
        return {
            'total_items': self.metrics['total_items'],
            'total_updates': self.metrics['total_updates'],
            'total_queries': self.metrics['total_queries'],
            'type_distribution': self.metrics['type_distribution'],
            'source_distribution': self.metrics['source_distribution'],
            'status_distribution': self.metrics['status_distribution']
        }
    
    def configure(self, config: Dict[str, Any]):
        """配置组件参数"""
        self.config.update(config) 